import os, shutil, sys
from .        import taln_archives_parser as parser
from .Event   import Event
from .Article import Article
from .Author  import Author
from datetime import datetime
from tqdm     import tqdm

months = { 1:'January', 2:'February', 3:'March', 4:'April', 5:'May', 6:'June',
           7:'July', 8:'August', 9:'September', 10:'October', 11:'November',
           12:'December' }

volumes = [ (u'long', 'Articles longs'), 
            (u'court', 'Articles courts'),
            (u'orig', 'Travaux originaux'),
            (u'parallel', 'Traductions d\'articles en cours de soumission dans des conférences internationales'),
            (u'resume', 'Résumés d\'articles publiés dans des conférences internationales'),
            (u'survey', 'Prises de position'),
            (u'papers', 'Articles sélectionnés'),
            (u'posters', 'Posters'),
            (u'poster', 'Posters'),            
            (u'articles', 'Articles'),
            (u'démonstration', 'Démonstrations'),
            (u'demonstration', 'Démonstrations'),
            (u'demo', 'Démonstrations'),
            (u'jep', 'Journées d\'Études sur la Parole'),
            (u'inter', 'Résumés d\'articles internationaux'),
            (u'tutoriel', 'Tutoriels'),
            (u'recital', 'REncontres jeunes Chercheurs en ' + \
             'Informatique pour le Traitement Automatique des Langues'),
            (u'invite', 'Conférences invitées'),
            (u'recitalposter', 'REncontres jeunes Chercheurs en ' + \
             'Informatique pour le Traitement Automatique des Langues (Posters)'),
            (u'recitalcourt', 'REncontres jeunes Chercheurs en ' + \
             'Informatique pour le Traitement Automatique des Langues (articles courts)'),
]

def read_xml(event, where, xmlfile):
    ## read input file
    pdf_dir  = os.path.join(os.path.dirname(xmlfile), 'actes')
    ## Conference data
    conf     = parser.content_handler(xmlfile)
    event.__dict__['title']     = conf.meta['conf_titre']
    event.__dict__['abbrev']    = conf.meta['acronyme'].split("'")[0]
    if 'anthology_id' not in event.__dict__.keys() or event.__dict__['anthology_id'] == "":    
        event.__dict__['anthology'] = conf.meta['acronyme'].split("'")[0].lower()
    event.__dict__['year']      = conf.meta['acronyme'].split("'")[1]
    general_chairs = filter(lambda x : x[2] is None, conf.meta['presidents'])
    event.__dict__['chairs']    = list(map(lambda x : x[1] + ', ' + x[0], \
                                           general_chairs))
    event.__dict__['booktitle'] = 'Actes de la ' + event.__dict__['title']
    event.__dict__['shortbooktitle'] = 'Actes de '+ conf.meta['acronyme']
    event.__dict__['location']  = conf.meta['ville']+', '+conf.meta['pays']
    event.__dict__['month']     = months[int(conf.meta['dateDebut'].split('-')[1])]
    event.__dict__['begin']     = datetime.strptime(conf.meta['dateDebut'], '%Y-%m-%d')
    event.__dict__['end']       = datetime.strptime(conf.meta['dateFin'], '%Y-%m-%d')
    event.__dict__['publisher'] = 'ATALA'
    event.__dict__['url']       = conf.meta['siteWeb']
    event.__dict__['tracks']    = {}
    suffix                      = 1
    id_shift                    = 0 # to give distinct paper id in tracks
    for t in conf.meta['typeArticles']:
        track                   = {}
        track['volume']         = t[0]
        if t[0] in dict(volumes):
            track['fullname']   = dict(volumes)[t[0]] #cf taln_parser
        else:
            print("\n[Warning] Unknown full track name for track: " + t[0], \
                  file=sys.stderr)
            track['fullname']   = t[1]
        suffix                 += 1
        track_chairs           = list(filter(lambda x : x[2] is not None and \
                                             t[0] in x[2].split(","), \
                                             conf.meta['presidents']))
        if len(track_chairs) > 0:
            track['chairs']    = ", ".join(list(map(lambda x : x[0] + ' ' + x[1], \
                                                    track_chairs))) 
        event.__dict__['tracks'][t[0]] = track 
    ## Article and authors data for each track
    #for vol_type, vol_title in volumes:
        vol_type = t[0]
        vol_title= dict(volumes)[t[0]] if t[0] in dict(volumes) else t[1]
        id_shift+= 1000 #to build unique paper ids (max. 1000 papers per track)
        articles = [] 
        authors  = {}
        vol_articles = [u for u in conf.articles if u['type'] == vol_type]
        if len(vol_articles) > 0:
            print('Processing track:', vol_title, file=sys.stderr)
        for article in tqdm(vol_articles):
            #print(article)
            paper = Article()
            archives_artid = article['id'] ## of the form taln-2008-long-001
            archives_id    = str(id_shift + \
                                 int(archives_artid[-3:])) ## new unique id of the form 1001
            new_filename   = event.__dict__['abbrev'] + '_' + \
                             event.__dict__['year'] + '_paper_' + archives_id
            if os.path.exists(os.path.join(pdf_dir, archives_artid+'.pdf')):
                shutil.copy(os.path.join(pdf_dir, archives_artid+'.pdf'), \
                            os.path.join(pdf_dir, new_filename+ '.pdf'))
            else:
                shutil.copy(os.path.join(where, 'tex', 'blank.pdf'), \
                            os.path.join(pdf_dir, new_filename+ '.pdf'))
            paper.__dict__['paperid']= archives_id
            paper.__dict__['url']    = os.path.join(os.getcwd(), pdf_dir, new_filename+'.pdf')
            paper.__dict__['track']  = vol_type
            aut = []
            affiliations = dict(article['affiliations'])
            #print(article['affiliations'])
            aut_pos = 0
            for prenom, nom, rank, email in article['auteurs']:
                aut_pos+= 1
                author  = Author()
                author.__dict__['firstname']  = prenom
                author.__dict__['lastname']   = nom
                author.__dict__['email']      = email
                author.__dict__['affiliation']= affiliations[rank]
                author.__dict__['paperid']    = paper.__dict__['paperid']
                author.__dict__['rank']       = aut_pos
                aut.append(prenom.strip() + ' ' + nom.strip())
                if paper.__dict__['paperid'] not in authors.keys():
                    authors[paper.__dict__['paperid']] = []
                authors[paper.__dict__['paperid']].append(author)
            paper.__dict__['authors'] = ', '.join(aut)
            if article['title'] == article['titre'] or article['titre'] == '':
                # the paper is in English
                paper.__dict__['title']    = article['title']
                paper.__dict__['language'] = 'en'
            else:
                paper.__dict__['title'] = article['titre']
                paper.__dict__['language'] = 'fr'
                paper.__dict__['title2'] = article['title']
                paper.__dict__['language2'] = 'en'
            if paper.__dict__['language'] == 'fr': 
                if article['mots_cles'] != '':
                    paper.__dict__['keywords'] = article['mots_cles']
                if article['resume'] != '':
                    paper.__dict__['abstract'] = article['resume']
            elif paper.__dict__['language'] == 'en':
                if article['keywords'] != '':
                    paper.__dict__['keywords'] = article['keywords']
                if article['abstract'] != '':
                    paper.__dict__['abstract'] = article['abstract']
            if article['pages'] != '':
                paper.__dict__['pages'] = article['pages']
            articles.append(paper)
        if len(articles) > 0:
            event.__dict__['tracks'][vol_type]['articles'] = articles
            event.__dict__['tracks'][vol_type]['authors']  = authors
    return event



class Article(object):

    def __init__(self):
        self.paperid   = ''
        self.title     = ''
        self.title2    = ''   #for second title
        self.language  = ''
        self.language2 = ''
        self.authors   = ''   #list of names
        self.abstract  = ''
        self.keywords  = ''
        self.pages     = ''   #of the form begin-end (used only for TALN import)
        self.numpages  = 0
        self.track     = ''
        self.session   = ''   #session id (related to paper type or topic)
        self.url       = ''
        self.idhal     = ''   #to specify an idhal in the article.csv file

    def __str__(self):
        s = ''
        for k,v in self.__dict__.items():
            s += k + ':' + str(v) + '\n'
        return s

    def update_title(self):
        if self.language == 'fr':
            if self.title2 != '':
                self.title = self.title + ' (' + self.title2 + ') [In French]' 
            else:
                self.title = self.title + ' [In French]'



import zipfile, sys, os, csv, re, json
from tqdm        import tqdm
from io          import StringIO
from unicodedata import normalize

def get_labs(s, affs):
    all_labs = re.findall(r'(\d+) - ([^\()]+) \(([^\)]+)\)', s)
    #print("**", all_labs)
    for x in all_labs:
        #print('**', x)
        idx, name, country = x
        affs[idx] = name + ', ' + country
        
def get_data(e, xzipfile, indir, verbose, xid_col, xauthors_col, xtitle_col, xidhal_col, xfile_col, xaccept_col, xkeywords_col):
    
    # Process the zip file to compute the expected tracks (csv and pdf dir/files) then call easychair interface
    articles    = ''  #csv input string
    zip_content = [] 
    eventname   = e.__dict__['abbrev'] + "_" + str(e.__dict__['year'])
    tracks_art  = {}  #track name <è> list of articles (each one being a csv string)
    tracks_abs  = {}  #track name <-> dict : paperid <-> abstract
    tracks_aut  = {}  #track name <-> dict : paperid <-> dict : rank <-> author

    ## read submissions.csv from zip file
    with zipfile.ZipFile(xzipfile, mode="r") as archive:
        #archive.printdir()
        articles = archive.read('export/submissions.csv').decode(encoding="utf8")        
        for filename in archive.namelist():
            zip_content.append(filename)
    
    # Fields in the submission page from easychair 
    PAPERINFO = ['DOCID', 'TYPE', 'DATE', 'STATUT','TYPDOC','ABSTRACT','TITLE',\
                 'SPEAKERS','CORRESPONDING', 'AUTHORS','LABOS','FILE','FILE_SRC','DATEPRODUCT','COMMENTAIRE',\
                 'LANGUE','CREATEUSERID','MAIL','TOPIC','MOTCLE','NOTE']

    # for each article retrieve the corresponding pieces of information
    csvfile= StringIO(articles)
    # get csv file length (stored in var count) and then go back to beginning of file
    reader = csv.reader(csvfile, delimiter='\t')
    count = sum(1 for _ in reader)
    csvfile.seek(0)
    reader = csv.reader(csvfile, delimiter='\t')
    # process articles (i.e. csv row)
    for row in tqdm(reader, total=count):
        #print("**",row)
        if not(row[0].isdigit()): #header row of csv file
            continue
        trackname= row[PAPERINFO.index('TYPDOC')].replace(' ', '-') # beware of spaces in dir names
        trackdir = os.path.join(indir, trackname)
        # the first time the track name is encountered
        # create the corresponding subdirectory
        if not(os.path.exists(trackdir)):
            os.makedirs(trackdir)
            os.makedirs(os.path.join(trackdir, 'pdf'))
        if trackname not in tracks_art:
            tracks_art[trackname] = []
            tracks_abs[trackname] = {}
            tracks_aut[trackname] = {}
        # we process the article info and format it as expected (compliently with config format)
        accept = row[PAPERINFO.index('STATUT')]
        # we create a list which will serve to compute the csv row of the processed article
        article = ['']*(max(xid_col, xauthors_col, xtitle_col, xidhal_col, xfile_col, xaccept_col, xkeywords_col)+1)
        # we update the article fields
        article[xid_col] = row[PAPERINFO.index('DOCID')]
        # we need to pre-process affiliations
        affiliations = row[PAPERINFO.index('LABOS')]
        laboratories = {} # key: int (rank), value: str (lab name)
        all_affs     = {} # key: str (author), value: str (lab name)
        get_labs(affiliations, laboratories) # update laboratories
        # we need to pre-process authors which are of the form "lastname~firstname <emails> (affiliation)"
        authors = row[PAPERINFO.index('AUTHORS')]
        auts    = ''
        rank    = 0
        for auth in authors.split(','):
            rank += 1
            aut = auth.strip().split('(')[0].split('<')[0]
            #print(article[xid_col], auth.strip())
            if '<' in auth:
                email = auth.strip().split('(')[0].split('<')[1].split('>')[0]
            else:
                email = ''
            lab = auth.strip().split('(')[1].split(')')[0] #only the main/first affiliation is stored
            #print(article[xid_col], lab)
            # we split names according to separator
            tokens = re.split('\xa0', aut) 
            if len(tokens) > 1:
                firstname = tokens[1].title().strip().replace(" ", "~")
                lastname  = tokens[0].strip().replace(" ", "~")
                newaut = firstname + ' ' + lastname
                #print("*",article[xid_col],newaut)
            else:
                newaut = aut
                #print("**",article[xid_col],newaut)         
            auts += newaut + ', ' #we compute the str representing the authors in the csv easychair input format
            #we prepare the dictionary of authors
            all_affs[rank] = { 'firstname': firstname, 'lastname': lastname, 'email': email, 'affiliation' : laboratories[lab] }
        # we update the tracks authors dict
        tracks_aut[trackname][article[xid_col]] = all_affs
        # we update article's metadata
        article[xauthors_col] = auts[:-2] # we remove the final ', '
        article[xtitle_col]   = row[PAPERINFO.index('TITLE')]
        #print("*", article[xtitle_col])
        article[xkeywords_col]= row[PAPERINFO.index('MOTCLE')]
        article[xidhal_col]   = '' # ignored at this stage
        if accept.startswith('Accept'):
            article[xaccept_col] = accept  
        elif accept.startswith('Initial'):
            article[xaccept_col] = 'Accept' 
        else:
            article[xaccept_col] = 'No'
        # we update the abstracts info
        tracks_abs[trackname][article[xid_col]] = { 'abstract' : row[PAPERINFO.index('ABSTRACT')].replace('\n',''), 'title' : article[xtitle_col] }
        #print("#", [article[xid_col]], row[PAPERINFO.index('ABSTRACT')].replace('\n',''))
        ## copy pdf file
        paperid = article[xid_col]
        p = re.compile("export/" + paperid + ".pdf")
        pdffile = list(filter(lambda x : p.match(x), zip_content))[0]
        newname = eventname + "_" + paperid + ".pdf"
        with zipfile.ZipFile(xzipfile, mode="r") as archive2:
            archive2.extract(pdffile, path=os.path.join(trackdir, 'pdf'))
        os.rename(os.path.join(trackdir, 'pdf', pdffile), \
                os.path.join(trackdir, 'pdf', newname))
        # we store the new filename                      
        article[xfile_col] =  newname
        # we convert current article into a csv entry 
        # and add it to the list of papers
        tracks_art[trackname].append('\t'.join(article))
    # finally we write down articles.csv, abstracts.csv and authors.json
    for tname in tracks_art:
        with open(os.path.join(indir, tname, 'articles.csv'), mode='w') as file_art:
            print('\n'.join(tracks_art[tname]), file=file_art)
            if verbose > 1:
                print('\n'.join(tracks_art[tname]), file=sys.stdout)
        with open(os.path.join(indir, tname, 'abstracts.json'), mode='w') as file_abs:
            print(json.dumps(tracks_abs[tname]), file=file_abs)
            if verbose > 1:
                print(json.dumps(tracks_abs[tname]), file=sys.stdout)
        with open(os.path.join(indir, tname, 'authors.json'), mode='w') as file_aut:
            print(json.dumps(tracks_aut[tname]), file=file_aut)
            if verbose > 1:
                print(json.dumps(tracks_aut[tname]), file=sys.stdout)



import os, sys, shutil, re, zipfile, xmlschema, tqdm, subprocess, html
import xml.etree.cElementTree as ET
from xml.dom               import minidom
from unidecode             import unidecode
from pybtex.database       import BibliographyData, Entry
from pycountry             import countries
from urllib.parse          import quote
from nameparser            import HumanName
from pylatexenc.latex2text import LatexNodes2Text
from country_list          import countries_for_language
from ..exports.anthology   import order_papers, order_session
from ..exports.archives    import BASE_URL


def get_iso_code(name, verbose=0):
    code = None
    countries_en = {v:k for k,v in dict(countries_for_language('en')).items()}
    countries_fr = {v:k for k,v in dict(countries_for_language('fr')).items()}
    try: #Try 1: English country name
      code = countries_en[name]
    except KeyError:
        try: # Try 2: French country name
            code = countries_fr[name]
        except KeyError:
            print('[Warning] Unknown country name: ' + name, file=sys.stderr)
    return code


def write_hal_xml(event, stopwords, verbose, behalf, stamp, include_pdf, guess, dry, \
                  update, sessions, no_meta, national):
    indir    = os.getcwd()
    outdir   = os.path.join(indir, 'out')
    # Preparing output files
    eventname= 'hal_' + event.__dict__['abbrev'] + \
               '-' + str(event.__dict__['year'])
    if not os.path.exists(os.path.join(outdir, eventname)):
        os.mkdir(os.path.join(outdir, eventname))
    ## Process articles
    for t in event.__dict__['tracks']:
        print('Processing track:', t, file=sys.stderr)
        if not os.path.exists(os.path.join(outdir, eventname, t)):
            os.mkdir(os.path.join(outdir, eventname, t))
        # Get pdf url (default is TALN archives)
        pdf_url = os.path.join(BASE_URL, event.__dict__['abbrev'], \
                                event.__dict__['abbrev'] +'-'+ str(event.__dict__['year']))
        if event.__dict__['tracks'][t].base_url != '':
            if not(event.__dict__['tracks'][t].__dict__['base_url'].startswith('http')):
                pdf_url = os.path.join(BASE_URL, event.__dict__['tracks'][t].__dict__['base_url'])
            else:
                pdf_url = event.__dict__['tracks'][t].__dict__['base_url']
        else:
            if include_pdf:
                print('[Warning] Option --include-pdf used but no base_url '+\
                      'defined in track configuration (event.yml), ' +\
                      'default (TALN archives) URL is used for X2hal\'s bib import, ' +\
                      'please check output.', file=sys.stderr)
        # Check if the track's config includes a "onbehalf" field
        # to be used to set owner during HAL export
        onbehalf = behalf #default (may be None)
        if hasattr(event.__dict__['tracks'][t],'onbehalf'):
            onbehalf = event.__dict__['tracks'][t].__dict__['onbehalf']
        behalfof = [('x-onbehalfof', onbehalf)] if onbehalf is not None else []
        publisher= [('publisher',   event.__dict__['publisher'])] \
            if event.__dict__['publisher'] != '' else []
        # stamp (collection) definition
        x_stamp = stamp #default (may be None)
        if hasattr(event.__dict__['tracks'][t],'stamp'):
            x_stamp = event.__dict__['tracks'][t].__dict__['stamp']
        x_stamp_tag = [('x-stamp', x_stamp)] if x_stamp is not None else []
        # Compiling track bib content
        country=event.__dict__['location'].split('(')[0].split(',')[1].strip()
        bibfile  = os.path.join(outdir, eventname, \
                                eventname.lower() + '-' + t.lower() + '.bib')
        bibfileh = open(bibfile, 'wt')
        ttitle   = event.__dict__['booktitle'] + '. ' + \
                   event.__dict__['tracks'][t].__dict__['fullname'].replace('\\', '')
        tchairs  = event.__dict__['chairs']
        if event.__dict__['tracks'][t].chairs != '':
            tchairs = list(map(lambda y : HumanName(y.strip()).last+', '+\
                               HumanName(y.strip()).first,\
                               event.__dict__['tracks'][t].__dict__['chairs'].split(',')))
        #print(country, countries.get(name=country))
        bib_data = BibliographyData(
            { unidecode(event.__dict__['abbrev']) + \
              ':' + str(event.__dict__['year']) : \
              Entry('proceedings', [
                  ('editor',      ' and '.join(tchairs)),
                  ('title',       ttitle),
                  ('month',       str(event.__dict__['month'])),
                  ('year',        str(event.__dict__['year'])),
                  ('address',     event.__dict__['location']),
                  ('x-language',  'fr'),
                  ('x-audience',  'National' if national else 'International'),
                  ('x-popularlevel', 'No'),
                  ('x-city',      event.__dict__['location'].split(',')[0].strip()),
                  ('x-country',   get_iso_code(name=country) \
                   if get_iso_code(name=country) is not None \
                   else 'FR'),
                  ('x-conferencestartdate', event.__dict__['begin'].isoformat()),
                  ('url',         event.__dict__['url']),
              ] + behalfof + x_stamp_tag + publisher),
            })
        bibfileh.write(bib_data.to_string('bibtex'))
        bibfileh.write('\n\n')
        ## Process paper in right order (cf automatic numbering)
        papers  = event.__dict__['tracks'][t].__dict__['articles']
        # track's config overrides command line one
        start   = 1 #default
        if int(event.__dict__['tracks'][t].__dict__['startpage']) > 0:
            start = int(event.__dict__['tracks'][t].__dict__['startpage'])
        order   = 'input' #default
        if event.__dict__['tracks'][t].order != '':
            order = event.__dict__['tracks'][t].__dict__['order']
        current_pos = start
        ordered = papers
        if sessions and event.__dict__['tracks'][t].sessions != '':
            ordered = order_session(papers, order, stopwords, verbose, \
                                    event.__dict__['tracks'][t])
        else:
            ordered = order_papers(papers, order, stopwords, verbose, \
                                   event.__dict__['tracks'][t].__dict__['authors'])
        for art in tqdm.tqdm(ordered):
            if art:
                write_hal_article(event, t, indir, os.path.join(outdir, eventname), \
                                  art, current_pos, bibfileh, behalfof, x_stamp_tag, \
                                  include_pdf, guess, dry, update, no_meta, pdf_url, national)
                current_pos += art.__dict__['numpages']
            else:
                current_pos += 1 #skip session's delimiter
        ## Close file handler and write down BIB content (cf X2Hal)
        bibfileh.close()
        ## Prepare shell script for upload to HAL via cURL
        shellfile = os.path.join(outdir, eventname, \
                                eventname.lower() + '-' + t.lower() + '.sh')
        shellfileh= open(shellfile, 'wt')
        logfile = os.path.join(eventname.lower() + '-' + t.lower() + '.log')
        curl = [f'#!/bin/bash\nrm -f {logfile}\ncd {t}\nfor i in $(ls *.sh)\ndo\n',
                f'\techo Processing article and file ${{i}} >> ../{logfile} ;\n'          
                f'\tbash ${{i}} >> ../{logfile} ;',
                f'\ndone\n']
        shellfileh.write(' '.join(curl))
        shellfileh.close()

def write_hal_article(event, track, indir, outdir, art, current_pos, bibfileh, behalf, \
                      stamptag, include_pdf, guess, dry, update, no_meta, base_url, national):
    paperid    = art.__dict__['paperid']
    xmlfile    = os.path.join(outdir, track, paperid + '.xml')
    #print('*', art)
    #print('**', list(map(lambda x: event.__dict__['tracks'][track].__dict__['authors'][paperid][x].__str__(), range(len(event.__dict__['tracks'][track].__dict__['authors'][paperid])))))
    # Preparing XML content (DOM)
    root       = ET.Element('TEI', xmlns="http://www.tei-c.org/ns/1.0")
    root.set("xmlns:hal","http://hal.archives-ouvertes.fr/")
    text       = ET.SubElement(root, 'text')
    body       = ET.SubElement(text, 'body')
    lbibl      = ET.SubElement(body, 'listBibl')
    bfull      = ET.SubElement(lbibl,'biblFull')
    # Processing paper
    ## Updating XML file
    ## Title Statement
    tstmt      = ET.SubElement(bfull, 'titleStmt')
    title      = ET.SubElement(tstmt, 'title')
    title.set('xml:lang', html.unescape(art.__dict__['language']))
    title.text = art.__dict__['title']
    title2     = ET.SubElement(tstmt, 'title')
    title2.set('xml:lang', html.unescape(art.__dict__['language2']))
    title2.text= art.__dict__['title2']
    affilia    = [] #name, country
    autbib     = [] #authors as bibtex strings
    try:
        for au in event.__dict__['tracks'][track].__dict__['authors'][paperid]:
            auteur  = ET.SubElement(tstmt, 'author', role="aut")
            autbib.append((re.sub(r'([^\\])~', r'\1 ', au.__dict__['firstname']), \
                           re.sub(r'([^\\])~', r'\1 ', au.__dict__['lastname'])))
            pers    = ET.SubElement(auteur, 'persName')
            ET.SubElement(pers, 'forename', type='first').text = au.__dict__['firstname'].replace('~', ' ')
            ET.SubElement(pers, 'surname').text                = au.__dict__['lastname'].replace('~', ' ')
            ET.SubElement(auteur,'affiliation',ref="#localStruct-"+str(au.__dict__['rank']))
            affilia.append(au.__dict__['affiliation'])
    except KeyError: #paperid not found in track's authors (cf author_list.xlsx)
        print(f'[Warning] author unknown for paper {paperid}', file=sys.stderr)
        for aut2 in art.__dict__['authors'].replace(' and ', ', ').split(', '):
            aut     = re.sub(r'([^\\])~', r'\1 ', aut2) #replace unsplittable spaces
            auteur  = ET.SubElement(tstmt, 'author', role="aut")
            #fname  = aut.split(' ')[0]
            #lname  = ' '.join(aut.split(' ')[1:])
            fname   = HumanName(aut.strip()).first + ' ' + HumanName(aut.strip()).middle
            lname   = HumanName(aut.strip()).last
            autbib.append((fname, lname))
            pers    = ET.SubElement(auteur, 'persName')
            ET.SubElement(pers, 'forename', type="first").text = fname
            ET.SubElement(pers, 'surname').text                = lname
    ## Edition Statement
    estmt  = ET.SubElement(bfull, 'editionStmt')
    edition= ET.SubElement(estmt, 'edition')
    if include_pdf:
        ET.SubElement(edition, 'ref', type='file', subtype='greenPublisher', n='1', \
                      target=paperid + '.pdf')
    ## Publication Statement
    pstmt = ET.SubElement(bfull, 'publicationStmt')
    avail = ET.SubElement(pstmt, 'availability')
    ET.SubElement(avail, 'licence', target="https://creativecommons.org/licenses/by/")
    ## Series Statement
    sstmt = ET.SubElement(bfull, 'seriesStmt')
    if stamptag != []:
        for x in stamptag[0][1].split(';'):
            ET.SubElement(sstmt, 'idno', type="stamp", n=x)
    ## Notes Statement
    audience_code = "3" if national else "2"
    nstmt = ET.SubElement(bfull, 'notesStmt')
    ET.SubElement(nstmt, 'note', type="audience",   n=audience_code) 
    #1 => unspecified, 2 => international, 3 => national
    ET.SubElement(nstmt, 'note', type="invited",    n="0")
    ET.SubElement(nstmt, 'note', type="popular",    n="0")
    ET.SubElement(nstmt, 'note', type="peer",       n="1")
    ET.SubElement(nstmt, 'note', type="proceedings",n="1")
    ## Source Desc
    sdesc   = ET.SubElement(bfull, 'sourceDesc')
    bstruct = ET.SubElement(sdesc, 'biblStruct')
    analytic= ET.SubElement(bstruct, 'analytic')
    titleA  = ET.SubElement(analytic, 'title')
    titleA.set('xml:lang', art.__dict__['language'])
    titleA.text = art.__dict__['title']
    try:
        #print(paperid, sorted(event.__dict__['tracks'][track].__dict__['authors']))
        for au in event.__dict__['tracks'][track].__dict__['authors'][paperid]:
            auteurA  = ET.SubElement(analytic, 'author', role="aut")
            persA    = ET.SubElement(auteurA, 'persName')
            ET.SubElement(persA, 'forename', type='first').text = au.__dict__['firstname'].replace('~', ' ')
            ET.SubElement(persA, 'surname').text                = au.__dict__['lastname'].replace('~', ' ')
            ET.SubElement(auteurA, 'email').text = au.__dict__['email']
            ET.SubElement(auteurA, 'affiliation',ref="#localStruct-"+str(au.__dict__['rank']))
    except KeyError:
        print(f'[Warning] author unknown for paper {paperid}', file=sys.stderr)
        for aut2 in art.__dict__['authors'].replace(' and ', ', ').split(', '):
            aut      = re.sub(r'([^\\])~', r'\1 ', aut2)
            auteurA  = ET.SubElement(analytic, 'author', role="aut")
            #fname   = aut.split(' ')[0]
            #lname   = ' '.join(aut.split(' ')[1:])
            fname    = HumanName(aut.strip()).first + ' ' + HumanName(aut.strip()).middle
            lname    = HumanName(aut.strip()).last
            persA    = ET.SubElement(auteurA, 'persName')
            ET.SubElement(persA, 'forename', type="first").text = fname
            ET.SubElement(persA, 'surname').text                = lname
    monog = ET.SubElement(bstruct, 'monogr')
    ET.SubElement(monog, 'title', level="m").text = event.__dict__['booktitle'] + '. ' + \
                                                    event.__dict__['tracks'][track].__dict__['fullname'].replace('\\', '')
    meetin= ET.SubElement(monog, 'meeting')
    ET.SubElement(meetin, 'title').text = event.__dict__['title'].replace('\\', '-')
    ET.SubElement(meetin, 'date', type="start").text = str(event.__dict__['year'])
    ET.SubElement(meetin, 'settlement').text = event.__dict__['location'].split(',')[0]
    country = event.__dict__['location'].split('(')[0].split(',')[1].strip()
    #print(country,  countries.get(name=country))
    ET.SubElement(meetin, 'country', key=get_iso_code(name=country))
    tchairs = event.__dict__['chairs']
    if event.__dict__['tracks'][track].chairs != '':
        tchairs = list(map(lambda y : HumanName(y.strip()).last+', '+\
                            HumanName(y.strip()).first,\
                            event.__dict__['tracks'][track].__dict__['chairs'].split(',')))
    for c in tchairs:
        ET.SubElement(monog, 'editor').text = c
    imprint = ET.SubElement(monog, 'imprint')
    ET.SubElement(imprint, 'publisher').text = event.__dict__['publisher']
    pages = art.__dict__['pages']
    if pages == '':
        pages = str(current_pos) + '-' + str(current_pos + art.__dict__['numpages'] - 1)
    ET.SubElement(imprint, 'biblScope', unit="pp").text = pages
    ET.SubElement(imprint, 'date', type="datePub").text = str(event.__dict__['year'])
    ## profileDesc
    pdesc = ET.SubElement(bfull, 'profileDesc')
    lusage= ET.SubElement(pdesc, 'langUsage')
    lang  = art.__dict__['language'] if art.__dict__['language'] == 'fr' else 'en' 
    #lang defined via an if expression due to (rare) wrong lang detect
    ET.SubElement(lusage, 'language', ident=lang)
    tclass= ET.SubElement(pdesc, 'textClass')
    kwords= ET.SubElement(tclass, 'keywords', scheme="author")
    for k in art.__dict__['keywords'].split(','):
        kw = ET.SubElement(kwords, 'term')
        kw.text = k.strip()
        kw.set('xml:lang', lang)
    ET.SubElement(tclass, 'classCode', scheme="halDomain", n="info.info-cl")
    ET.SubElement(tclass, 'classCode', scheme="halTypology", n="COMM")
    abstract = ET.SubElement(pdesc, 'abstract')
    abstract.text = art.__dict__['abstract']
    abstract.set('xml:lang', lang)
    ## Back
    back = ET.SubElement(text, 'back')
    lorg = ET.SubElement(back, 'listOrg', type="structures")
    #print('**',affilia)
    if len(affilia) > 0:
        for aff in range(len(affilia)):
            affname   = ', '.join(affilia[aff].split(',')[:-1]).strip()
            affcountry= affilia[aff].split(',')[-1].strip()
            org       = ET.SubElement(lorg, 'org', type="institution")
            org.set('xml:id', "localStruct-" + str(aff+1))
            ET.SubElement(org, 'orgName').text = affname
            desc      = ET.SubElement(org, 'desc')
            address   = ET.SubElement(desc, 'address')
            #print('**',affcountry, art.__dict__['paperid'], get_iso_code(affcountry))
            real_affcountry = affcountry.replace('South Korea','Korea, Republic of').replace('République tchèque', 'Tchéquie')
            ET.SubElement(address, 'country', key=get_iso_code(real_affcountry))
    else:
        print(f'[Warning] affiliations not found for paper {paperid}', file=sys.stderr)
        ET.SubElement(lorg, 'org', type="institution")
    ## Generate article bib file (and populating event bib file)
    #  Get title Language
    title2label = 'x-title_en' if lang == 'fr' else 'x-title_fr'
    #  Include pdf if required
    pdf_info = []
    if include_pdf:
        pdf_info = [('pdf', os.path.join(base_url, paperid +'.pdf')),
                    ('x-filesource', 'greenPublisher')]
    bib_key = '-'.join(list(map(lambda x: unidecode(x[1].replace(' ', '-')), autbib[:3]))) + \
              '-p' + pages.split('-')[0] + \
              ':' + unidecode(event.__dict__['abbrev']) + ':' + \
              str(event.__dict__['year'])
    tchairs  = event.__dict__['chairs']
    publisher= [('publisher',   event.__dict__['publisher'])] \
        if event.__dict__['publisher'] != '' else []
    if event.__dict__['tracks'][track].chairs != '':
        tchairs = list(map(lambda y : HumanName(y.strip()).last+', '+\
                           HumanName(y.strip()).first,\
                           event.__dict__['tracks'][track].__dict__['chairs'].split(',')))
    bib_art = BibliographyData({
        bib_key : Entry('inproceedings', [
            ('author',    ' and '.join(list(map(lambda x: x[1]+', '+x[0],autbib)))),
            ('title',     art.__dict__['title']),
            ('booktitle', event.__dict__['booktitle'] + '. ' + \
             event.__dict__['tracks'][track].__dict__['fullname'].replace('\\', '')),
            ('editor',    ' and '.join(tchairs)),
            ('month',     str(event.__dict__['month'])),
            ('year',      str(event.__dict__['year'])),
            ('address',   event.__dict__['location']),
            ('pages',     pages),
            (title2label, art.__dict__['title2']), 
            ('abstract',  art.__dict__['abstract']),
            ('keywords',  art.__dict__['keywords'].replace(',', ';')),
            ('x-language',lang),
            ('x-audience','National' if national else 'International'),
            ('x-peerreviewing','Yes'),
            ('x-popularlevel', 'No'),
            ('x-invitedcommunication', 'No'),
            ('x-proceedings',  'Yes'),
            ('x-city',    event.__dict__['location'].split(',')[0].strip()),
            ('x-country', get_iso_code(name=country)),
            ('x-conferencestartdate', event.__dict__['begin'].isoformat()),
        ] + behalf + publisher + stamptag + pdf_info),
    })
    bibfileh.write(bib_art.to_string('bibtex'))
    bibfileh.write('\n\n')
    #debug only
    #print(bib_art.to_string('bibtex'))
    xmlstr=''
    if update and no_meta:
        cmd   = ["curl", "-s", \
                 'https://api.archives-ouvertes.fr/search/?q=title_s:\"'\
                 + quote(LatexNodes2Text().latex_to_text(art.__dict__['title']))\
                 .replace('%C2%A0', '%20').replace('%20%20%20', '%26')\
                 + '\"&wt=xml-tei&fl=*']
        #print(' '.join(cmd))
        out   = subprocess.check_output(cmd)
        root2 = ET.fromstring(out.decode())
        ns    = {'ns0':"http://www.tei-c.org/ns/1.0"}
        root2.remove(root2.find('ns0:teiHeader', ns))
        # In case the search is sucessful:
        bibf  = root2.find('.//ns0:biblFull', ns)
        if bibf is not None: #there is a result (should be unique cf search by exact title)
            # Clear tags to be ignored
            dep = root2.find (".//ns0:editor[@role='depositor']", ns)
            if dep is not None:
                dep.clear()
            edi = root2.find('.//ns0:editionStmt', ns)
            if edi is not None:
                edi.clear()
                ed = ET.SubElement(edi, 'edition')
                # Attach new pdf version
                if include_pdf:
                    ET.SubElement(ed, 'ns0:ref', type='file', subtype='greenPublisher', n='1', \
                                  target=paperid + '.pdf')
            # Update licence (mandatory to fix previous exports)
            licen = root2.find('.//ns0:licence', ns)
            if licen is not None :
                licen.clear()
                root2.find('.//ns0:licence', ns).set('target',"https://creativecommons.org/licenses/by/")
            else:
                pubs = root2.find('.//ns0:publicationStmt', ns)
                avai = ET.SubElement(pubs, 'ns0:availability')
                ET.SubElement(avai, 'ns0:licence', target="https://creativecommons.org/licenses/by/")
            # Fix idno for authors
            auth = root2.findall('.//ns0:author', ns)
            for au in auth:
                keep      = True
                to_remove = []
                for child in au:
                    if child.tag == '{http://www.tei-c.org/ns/1.0}affiliation':
                        keep = False
                    elif child.tag == '{http://www.tei-c.org/ns/1.0}idno':
                        if not keep:
                            to_remove.append(child)
                    elif child.tag == '{http://www.tei-c.org/ns/1.0}email':
                        to_remove.append(child) #email received by HAL search are ill-formed
                for i in range(len(to_remove)):
                    x = to_remove.pop()
                    au.remove(x)
        else:
            print('\n[Warning] No reference found in HAL for paper ' + art.__dict__['paperid'] + \
                  ' (article skipped)\n', file=sys.stderr) 
            return #move to next article
        # Write XML to string (while removing ns0 namespace prefix)
        root2.set("xmlns:hal","http://hal.archives-ouvertes.fr/")
        xmlstr= minidom.parseString(ET.tostring(root2, encoding='utf-8', 
                                                short_empty_elements=False)).\
                                                toprettyxml(indent='   ').\
                                                replace(':ns0', '').replace('ns0:', '')
        #print(xmlstr)
    else:
        ## Write down XML file
        #xmls   = ET.tostring(root, encoding='utf-8', short_empty_elements=False).decode()
        #print(xmls)
        xmlstr = minidom.parseString(ET.tostring(root, encoding='utf-8', 
                                                 short_empty_elements=False)).\
                                                 toprettyxml(indent='   ')
    with open(xmlfile, 'w') as f:
        f.write(xmlstr)
    ## Validate XML file
    schema = xmlschema.XMLSchema(os.path.join(indir, 'out', 'aofr.xsd'))
    try:
        schema.validate(xmlfile)
    except xmlschema.validators.exceptions.XMLSchemaChildrenValidationError as xe:
        print('XML schema validation error', file=sys.stderr)
        print(xe, file=sys.stderr)
        sys.exit(1)
    ## Copy pdf file and zip it together with XML 
    if include_pdf:
        if os.path.exists(os.path.join(outdir, track, paperid + '.pdf')):
            os.remove(os.path.join(outdir, track, paperid + '.pdf'))
        pdf_indir = os.path.join(indir, track, 'pdf')
        evtname   = event.__dict__['abbrev'] + '-' + str(event.__dict__['year'])
        #print('**', os.path.join(indir, 'out', 'pdf_'+evtname, track, 'pdf_out'))
        if os.path.exists(os.path.join(indir, 'out', 'pdf_'+evtname, track, 'pdf_out')):
            pdf_indir = os.path.join(indir, 'out', 'pdf_'+evtname, track, 'pdf_out')
        st = os.stat(os.path.join(pdf_indir, os.path.basename(art.__dict__['url'])))
        shutil.copy(os.path.join(pdf_indir, os.path.basename(art.__dict__['url'])), \
                    os.path.join(outdir, track, paperid + '.pdf'))
        os.chmod(os.path.join(outdir, track, paperid + '.pdf'), st.st_mode)

        # Zip file cf 
        # https://stackoverflow.com/questions/1855095/
        # how-to-create-a-zip-archive-of-a-directory-in-python
        zipf = zipfile.ZipFile(os.path.join(outdir, track, paperid + '.zip'), \
                               'w', zipfile.ZIP_DEFLATED)
        xmlfile = os.path.join(outdir, track, paperid + '.xml')
        pdffile = os.path.join(outdir, track, paperid + '.pdf')
        zipf.write(xmlfile, os.path.basename(xmlfile))
        zipf.write(pdffile, os.path.basename(pdffile))
    ## Write down shell script for upload via cURL
    shellfile = os.path.join(outdir, track, paperid + '.sh')
    shellfileh= open(shellfile, 'wt')
    serv = 'https://api.archives-ouvertes.fr'
    opt  = '-H "X-test:1"' if dry else ''
    beh  = '-H "On-Behalf-Of: ' + \
           ';'.join(list(map(lambda x: 'idhal|' + x, behalf[0][1].split(';')))) \
           + '"' if len(behalf) > 0 else ''
    getid= f'ID=$(curl -s \'{serv}/search/?q=title_s:\"' \
                 + quote(LatexNodes2Text().latex_to_text(art.__dict__['title']))\
                 .replace('%C2%A0', '%20').replace('%20%20%20', '%26') \
                 + '\"&fl=halId_s\' | jq .response.docs[0].halId_s' \
                 + '| sed -e \'s/\"//g\'' + ')' \
                 if update else ''
    pid  = f'\necho Found ID: "${{ID}}"\n' if update else ''
    rtype= '-X PUT' if update else '-X POST'
    curl = []
    if include_pdf:
        gue  = '-H "X-Allow-Completion: grobid,affiliation"' if guess else '-H "LoadFilter: noaffiliation"'
        suf  = f'${{ID}}' if update else 'hal/'
        curl = [f'#!/bin/bash\n\n{getid}\n{pid}\n',
                f'\ncurl {rtype} -v -u $LOGIN:$PASSWORD',
                f'{serv}/sword/{suf} {beh} {gue} {opt}',
                f'-H "Packaging:http://purl.org/net/sword-types/AOfr"',
                f'-H "Content-Type:application/zip"',
                f'-H "ForceDoublonByTitle: 1"', #we solve duplicates a posteriori (merging of deposits)
                f'-H "Content-Disposition: attachment; filename={paperid}.xml"', 
                f'--data-binary @{paperid}.zip\n']        
    else:
        gue  = '-H "X-Allow-Completion: affiliation"' if guess else '-H "LoadFilter: noaffiliation"'
        vers = f'VERSION=$(curl -s \'{serv}/search/?q=title_s:\"' \
                 + quote(LatexNodes2Text().latex_to_text(art.__dict__['title']))\
                 .replace('%C2%A0', '%20').replace('%20%20%20', '%26') \
                 + '\"&fl=version_i\' | jq .response.docs[0].version_i' \
                 + ' )' \
                 if update else ''
        suf  = f'${{ID}}v${{VERSION}}' if update else 'hal/'
        curl = [f'#!/bin/bash\n\n{getid}\n{vers}\n{pid}\n',
                f'\ncurl {rtype} -v -u $LOGIN:$PASSWORD',
                f'{serv}/sword/{suf} {beh} {gue} {opt}',
                f'-H "Packaging:http://purl.org/net/sword-types/AOfr"',
                f'-H "Content-Type:text/xml"',
                f'-H "ForceDoublonByTitle: 1"', #we solve duplicates a posteriori (merging of deposits)
                f'-d @{paperid}.xml\n']
    shellfileh.write(' '.join(curl))
    shellfileh.close()



import os, sys, csv, shutil, subprocess, re
import locale, threading
from contextlib  import contextmanager
from distutils   import dir_util
from unidecode   import unidecode
from .           import Event, Article, Author
from nameparser  import HumanName

def clean_easy2acl(where):
    tmpfiles = ['meta', 'submissions', 'submission.csv', 'accepted']
    tmpdir   = ['pdf', 'book-proceedings', 'proceedings']
    for f in tmpfiles: 
        try:
            os.remove(os.path.join(where, f))
        except FileNotFoundError as e:
            pass
    for d in tmpdir:
        try:
            shutil.rmtree(os.path.join(where, d))
        except FileNotFoundError as e:
            pass
    if not os.path.exists(os.path.join(where, 'book-proceedings')):
        os.mkdir(os.path.join(where, 'book-proceedings'))
    if not os.path.exists(os.path.join(where,'pdf')):
        os.mkdir(os.path.join(where,'pdf'))

def order_session(papers, order, stopwords, verbose, track):
    ordered = []
    for s in track.__dict__['sessions'].split(';'):
        #collect corresponding papers
        spapers = list(filter(lambda x: x.__dict__['paperid'] in track.__dict__['partition'][s.strip()], \
                              papers))
        ospapers= order_papers(spapers, order, stopwords, verbose, track.__dict__['authors'])
        ordered.extend([None] + ospapers) #None is used as a session delimiter
    return ordered

def order_papers(papers, order, stopwords, verbose, tauthors):
    ordered = papers
    if order == 'author':
        try:
            all_authors = []
            for paper in ordered:
                pid = paper.__dict__['paperid']
                pauthors = ''.join(list(map(lambda x: x.__name4ordering__(), tauthors[pid])))
                all_authors.append((pid, pauthors))
            #print(all_authors)
            opauth   = sorted(all_authors, key=lambda z : z[1:]) #sort according to authors (not paperid)
            pdict    = {n.__dict__['paperid'] : n for n in papers} #turn papers into a dictionary
            ordered  = list(map(lambda x : pdict[x[0]], opauth))   #actual ordering
        except KeyError as ke:
            print('[Warning] Paper ' + str(ke) + ' not found in track\'s authors, '+\
                  'falling back on track\'s articles for ordering authors.', file=sys.stderr)
            ordered = sorted(papers, key=lambda x : 
                             ''.join(list(map(lambda y : (unidecode(HumanName(y.strip()).middle)+\
                                                          unidecode(HumanName(y.strip()).last)+\
                                                          unidecode(HumanName(y.strip()).first)),\
                                              x.__dict__['authors'].replace(' and ', ', ')\
                                              .split(',')))))
        #to debug only:               
        #for x in ordered:
        #    print('*', x.__dict__['authors'])
    elif order == 'title':
        if stopwords:
            import nltk
            from nltk.corpus import stopwords as sw
            for x in papers:
                if x.__dict__['language'] == 'fr':
                    wordpos  = 0
                    sentence = re.split('\W+', x.__dict__['title'])
                    while wordpos in range(len(sentence)) \
                          and sentence[wordpos].lower() in \
                          [x for x in sw.words('french') if x not in ['nous']]:
                        # nous should not be considered a stopword
                        wordpos += 1
                    x.__dict__['titleTmp'] = unidecode((' '.join(sentence[wordpos:])).lower())
                else:
                    x.__dict__['titleTmp'] = unidecode(x.__dict__['title'].lower())
                if verbose > 2:
                    print(x.__dict__['title'])
                    print(x.__dict__['titleTmp'])
            ordered = sorted(papers, key=lambda x : x.__dict__['titleTmp'])
        else:
            ordered = sorted(papers, key=lambda x : unidecode(x.__dict__['title']))
    return ordered


def easy2acl_event(where, event, track):
    ACLINFO = ['anthology', 'title', 'month', 'year', \
               'location', 'publisher', 'chairs', 'shortbooktitle']
    # Turn e into easy2acl's expected config file
    with open(os.path.join(where,'meta'), 'w') as outfile:
        for k,v in event.__dict__.items():
            if k in ACLINFO:
                if k == 'anthology':
                    print('abbrev' + ' ' + v.lower(), file=outfile)
                elif k == 'chairs': 
                    # if track has specific chairs, override general settings
                    if event.__dict__['tracks'][track].chairs != '':
                        for y in event.__dict__['tracks'][track].chairs.split(','):
                            print(k + ' ' + y.split()[1].strip() + ', ' + \
                                  y.split()[0].strip(), file=outfile)
                    else: #recall list of chairs are split over lines
                        for x in v:
                            print(k + ' ' + x.split(',')[0].strip() + ', ' + \
                                  x.split(',')[1].strip(), file=outfile)
                elif type(v) == list: 
                    for x in v:
                        print(str(k) + ' ' + str(v), file=outfile)
                else:
                    print(str(k) + ' ' + str(v), file=outfile)
        print('booktitle' + ' ' + event.__dict__['booktitle'] + '. ' +  \
              event.__dict__['tracks'][track].__dict__['fullname'], file=outfile)
        print('volume' + ' ' + event.__dict__['tracks'][track].__dict__['volume'], \
              file=outfile)
    if event.__dict__['anthology'] == '':
        print('Missing anthology id, cannot export to anthology format', file=sys.stderr)
        sys.exit(1)


def easy2acl_articles(where, e, papers, order, pdfdir, bilingual, stopwords, verbose, \
                      keep, sessions):
    # Start page (useful only in case of TALN import)
    taln_start= None
    # Ordered fields used in easy2acl
    SUBMISSIONS_PATTERN     = ['paperid', 'authors', 'title', 'url', 'keywords']
    SUBMISSIONS_CSV_PATTERN = ['paperid', 'title']
    trackname = os.path.basename(os.path.dirname(pdfdir))
    indir     = os.path.dirname(os.path.join(where, '..'))
    # Logging
    if keep:
        logfile = open(os.path.join(indir, trackname, 'submissions-'+ trackname +'.in.csv'),'w')
    with open(os.path.join(where,'submissions'), 'w') as outfile1:
        with open(os.path.join(where,'accepted'), 'w') as outfile2:
            with open(os.path.join(where,'submission.csv'), 'w') as abs_csv:
                writer = csv.writer(abs_csv)
                writer.writerow(['#', 'abstract'])
                ordered = papers
                if sessions and e.__dict__['tracks'][trackname].sessions != '':
                    ordered = order_session(papers, order, stopwords, verbose, \
                                            e.__dict__['tracks'][trackname])
                else:
                    ordered = order_papers(papers, order, stopwords, verbose, \
                                           e.__dict__['tracks'][trackname].__dict__['authors'])
                for article in ordered:
                    if article:
                        #print('***',article.__dict__['title'])
                        if taln_start is None and 'pages' in article.__dict__ \
                           and article.__dict__['pages'] != '':
                            taln_start = article.__dict__['pages'].split('-')[0]
                        if bilingual:
                            article.update_title()
                        writer.writerow([article.__dict__['paperid'],\
                                         article.__dict__['abstract']])
                        #for submissions, only the first 3 columns are read by easy2acl:
                        print('\t'.join(article.__dict__[x] \
                                        if type(article.__dict__[x]) == str \
                                        else ', '.join(article.__dict__[x]) \
                                        for x in SUBMISSIONS_PATTERN), \
                              file=outfile1) 
                        #for accepted, we need the status
                        # print ID TITLE [...] STATUS => outfile2
                        print('\t'.join(article.__dict__[x] for x in SUBMISSIONS_CSV_PATTERN)+\
                              '\t'+'ACCEPT', file=outfile2)
                        #copy pdf file
                        if os.path.exists(os.path.join(pdfdir, \
                                                       os.path.basename(article.__dict__['url']))):
                            if verbose > 1:
                                print('Copying pdf file ' + os.path.basename(article.__dict__['url']),\
                                      file=sys.stderr)
                            shutil.copy(os.path.join(pdfdir, \
                                                     os.path.basename(article.__dict__['url'])), \
                                        os.path.join(where,'pdf', \
                                                     e.__dict__['anthology'] + '_' + \
                                                     str(e.__dict__['year']) + '_paper_' + \
                                                     article.__dict__['paperid'] + '.pdf'))
                        #update log
                        if keep:
                            LOG_PATTERN=SUBMISSIONS_PATTERN[:3] +['track']+\
                                         [SUBMISSIONS_PATTERN[3]]+['title2','language','accept']+\
                                         [SUBMISSIONS_PATTERN[4]]
                            s = []
                            for y in LOG_PATTERN:
                                if y == 'url':
                                    s.append(os.path.basename(article.__dict__[y]))
                                elif y == 'accept':
                                    s.append('ACCEPT')
                                else:                            
                                    if type(article.__dict__[y]) == str :
                                        s.append(article.__dict__[y])
                                    else: 
                                        s.append(', '.join(article.__dict__[y]))
                            print('\t'.join(s),file=logfile) 
                    else: #session's delimiter
                        # we add an accepted article whose id is 0
                        # print ID TITLE [...] STATUS => outfile2
                        print('\t'.join(['-1', 'session delimiter'])+ \
                              '\t'+'ACCEPT', file=outfile2)
    #close log
    if keep:
        logfile.close()
    return taln_start


def easy2acl_export(e, bilingual, stopwords, verbose, keep, sessions, latex_encode):
    indir      = os.getcwd()
    outdir     = os.path.join(indir, 'out')
    where      = os.path.join(outdir, 'easy2acl')
    if not(os.path.exists(where)):
        os.makedirs(where)
        os.makedirs(os.path.join(where, 'pdf'))
        os.makedirs(os.path.join(where, 'pdf', 'book-proceedings'))
        os.makedirs(os.path.join(where, 'pdf', 'proceedings'))
        
    eventname  = e.__dict__['abbrev'] + '-' + str(e.__dict__['year'])            
    outdirname = os.path.join(outdir, 'acl_' + eventname)
    
    for track in e.__dict__['tracks'].keys():
        order = 'input' #default
        start = 1       #default
        clean_easy2acl(where)
        print('Exporting event information', file=sys.stderr)
        easy2acl_event(where, e, track)
        #print(track, e.__dict__['tracks'][track].keys())
        print('Processing track ' + e.__dict__['tracks'][track].__dict__['fullname'], file=sys.stderr)
        print('Exporting articles information', file=sys.stderr)
        # track's config overrides command line one
        if e.__dict__['tracks'][track].order != '':
            order = e.__dict__['tracks'][track].__dict__['order']
        pdf_indir = os.path.join(indir, track, 'pdf')
        if os.path.exists(os.path.join(indir, 'out', 'pdf_'+eventname, track, 'pdf_out')):
            pdf_indir = os.path.join(indir, 'out', 'pdf_'+eventname, track, 'pdf_out')
        tstart = easy2acl_articles(where, e, e.__dict__['tracks'][track].__dict__['articles'], order, \
                                   pdf_indir, bilingual, stopwords, \
                                   verbose, keep, sessions)
        # prepare tmp pdf file for frontmatter and full volume 
        # since these won't be compiled by easy2acl itself 
        if os.path.exists(os.path.join(indir, 'out', 'pdf_'+eventname, track, 'proceedings.pdf')):
            shutil.copy(os.path.join(indir, 'out', 'pdf_'+eventname, track, 'proceedings.pdf'), \
                        os.path.join(where, 'pdf', \
                                     e.__dict__['anthology'] + '_' + \
                                     str(e.__dict__['year']) + '.pdf'))
        else:
            shutil.copy(os.path.join(outdir,'blank.pdf'), \
                        os.path.join(where, 'pdf', \
                                     e.__dict__['anthology'] + '_' + \
                                     str(e.__dict__['year']) + '.pdf'))
        if os.path.exists(os.path.join(indir, 'out', 'pdf_'+eventname, track, 'frontmatter.pdf')):
            shutil.copy(os.path.join(indir, 'out', 'pdf_'+eventname, track, 'frontmatter.pdf'), \
                        os.path.join(where, 'pdf', \
                                     e.__dict__['anthology'] + '_' + \
                                     str(e.__dict__['year']) + \
                                     '_frontmatter.pdf'))
        else:
            shutil.copy(os.path.join(outdir,'blank.pdf'), \
                        os.path.join(where, 'pdf', \
                                     e.__dict__['anthology'] + '_' + \
                                     str(e.__dict__['year']) + \
                                     '_frontmatter.pdf'))

        print('Generating bibtex for articles using easy2acl', file=sys.stderr)
        # First call of easy2acl.py
        if tstart is not None: #read from input data (case of a TALN import)
            start = tstart
        curdir = os.getcwd()
        shutil.copy(os.path.join(outdir, 'easy2acl.py'), where)
        os.chdir(where)
        # Track configuration overwrite cli start option
        if int(e.__dict__['tracks'][track].startpage) > 0:
            start = int(e.__dict__['tracks'][track].startpage)
        try:
            returncode = subprocess.call([os.path.join('.','easy2acl.py'), str(start), \
                                          str(latex_encode)], timeout=10) 
            ## Copy easy2acl's proceedings dir
            dirname = e.__dict__['abbrev'] + str(e.__dict__['year']) + \
                      '-' + track
            if os.path.exists(os.path.join(curdir, outdirname + '_data', dirname)):
                shutil.rmtree(os.path.join(curdir, outdirname + '_data', dirname))
            if returncode == 0:
                shutil.copytree('proceedings', \
                                os.path.join(curdir, outdirname + '_data', dirname, \
                                             'proceedings'))
            else:
                print('''\nCreation of proceedings in ACL format failed, 
                please check easy2acl's above message\n''', file=sys.stderr)
        except subprocess.TimeoutExpired as toe:
            print('''Creation of proceedings in ACL format failed, 
            please check the input files''', file=sys.stderr)
        os.chdir(curdir)



import os, sys, csv, html, re, json
from openpyxl    import load_workbook
from unicodedata import normalize
from tqdm        import tqdm
from langdetect  import detect
from io          import StringIO
from bs4         import BeautifulSoup
from nameparser  import HumanName
from datetime    import datetime
from .           import Event, Track, Article, Author, readpdf
      
def get_articles(papers, abstracts, indir, accept, track_obj, \
                 verbose, ignore_pdf, sessions, \
                 id_col, authors_col, title_col, idhal_col, file_col, \
                 accept_col, keywords_col):
    # This function processes articles of a given track.
    # The accept parameter refers to the text to be used for acceptance 
    # (default is ACCEPT).
    all_papers      = [] #list of strings (output of the function)
    articles        = '' #proper csv string computed from the input file
    title2id        = {} #mapping title   <-> paperid
    title2abstract  = get_abstracts(abstracts)

    # [preprocessing] First, let us read the list of submitted papers
    # To fix keywords' format (newlines) to produce a proper csv string
    with open(papers, 'r') as f:
        current_article = None
        for line in f:
            line = normalize('NFC',line.strip())
            #print(line)
            if line and line[0].isdigit():
                if current_article is not None: 
                    # we store the article before moving on
                    articles += current_article + '\n'
                # we are moving to another article
                current_article = line
                #print(current_article)
                line_content = line.split('\t')
                # at the same time, we update the title2id dictionary
                # normalization wrt spaces
                # see https://stackoverflow.com/questions/2077897/
                # substitute-multiple-whitespace-with-single-whitespace-in-python
                norm_title = ' '.join(line_content[2].strip().lower().split())
                title2id[norm_title] = line_content[0].strip()
            else: 
                # lines which do not start with a number are keywords to
                # be appended to a paper description
                if line and line[0] != '#': # to ignore comments
                    current_article+= line
        # last entry should not be forgotten (since 
        # accumulation is triggered by new entries):
        if current_article:
            articles += current_article 
        #print(articles)

    # [processing] Second, let us read the computed proper csv string
    # and retrieve the corresponding info (e.g. pdf for the English title)
    reader = csv.reader(StringIO(articles), delimiter='\t', quotechar='|')
    for row in tqdm(reader, total=len(articles.split('\n'))):
        #print(PAPERINFO)
        #print(row)
        article = Article()
        patterns = accept.split(';') # in case there are several acceptance keywords
        if any(list(map(lambda acc : re.match('.*'+acc.strip(), row[accept_col], re.IGNORECASE), patterns))):
            ## update session info
            if sessions and track_obj.sessions != '':
                art_session = row[accept_col].split('_')[1]  # format: ACCEPT_SessionID
                if art_session in track_obj.sessions.split(';'):
                    if hasattr(track_obj,art_session): # to get session's full name
                        article.__dict__['session'] = track_obj.__dict__[art_session] 
                    else:
                        print(f'[Warning] {track_obj.trackid} full name unset in event.yml', \
                              file=sys.stderr)
                    track_obj.update_sessions(art_session, row[id_col])
                else:
                    if verbose > 0:
                        print('[Warning] Unknown session: ' + \
                              art_session, file=sys.stderr)
            ## collect paper info
            article.__dict__['paperid'] = row[id_col]
            article.__dict__['authors'] = row[authors_col]
            article.__dict__['title']   = row[title_col]
            article.__dict__['idhal']   = row[idhal_col]
            article.__dict__['keywords']= row[keywords_col]
            title                       = article.__dict__['title']
            title_norm                  = ' '.join(title.lower().split())
            title_norm                  = normalize('NFC', title_norm)
            if abstracts and title_norm in title2abstract:
                article.__dict__['abstract']= title2abstract[title_norm]
            else:
                #print("**",title2abstract.keys())
                #print("***", row[title_col])
                if verbose > 0:
                    print(f"[Warning] abstract not found for paper {article.__dict__['paperid']}", file=sys.stderr)   
            # make sure the halid info actually is a valid hal identifier
            midhal=re.fullmatch('hal-[0-9]*',article.__dict__['idhal'])
            if midhal is None:
                article.__dict__['idhal'] = ''
            # if authors are missing in articles.csv, try to get them from the abstracts' file
            if article.__dict__['authors'] == '': 
                try: 
                    article.__dict__['authors'] = id2authors[article.__dict__['paperid']]
                except KeyError as ke:
                    if verbose > 0:
                        print('[Warning] no author found for article ' + \
                              str(article.__dict__['paperid']), file=sys.stderr)
            # get article filename 
            article.__dict__['url'] = os.path.join(os.getcwd(), indir, 'pdf', row[file_col])
            #print(article.__dict__['url'])

            ## Detect the article's language from its title
            article.__dict__['language'] = detect(row[2])
            #print('***', article.__dict__['title'], article.__dict__['language'])

            ## Extract the article's secondary title from the pdf (if possible)
            pdf_path = os.path.join(indir, 'pdf', row[file_col])
            if not os.path.exists(pdf_path):
                print('[Error] PDF file not found: ' + pdf_path, file=sys.stderr)
                print('Please check input files.', file=sys.stderr)
                continue 
            # Read info from pdf file (article, indir, path, default_title, ignore_pdf, verbose)
            readpdf.read_pdf(article, indir, pdf_path, row[file_col], ignore_pdf, verbose)
            article.__dict__['track'] = track_obj.volume
            all_papers.append(article)
    return all_papers


def get_abstracts(abstracts):
    # Function which collects abstracts of a given track
    title2abstract = {}
    # read the abstracts (beware it is an html file)
    if abstracts:
        abstracts_type = os.path.splitext(abstracts)[1][1:] #do not forget to remove the dot in extension
        with open(abstracts,'r') as f:
            content = f.read()
            if abstracts_type == 'html':
                soup = BeautifulSoup(content, 'html.parser')
                #print(soup.prettify())
                for x in soup.find_all('div', attrs={'class':'paper'}):
                    authors = x.find('span', attrs={'class': 'authors'}).get_text().strip()[:-1] 
                    title   = x.find('span', attrs={'class': 'title'}).get_text().strip()
                    abstract= x.findNext('div').get_text()[10:]
                    title_norm = ' '.join(title.lower().split())
                    title_norm = normalize('NFC', title_norm)
                    title2abstract[title_norm] = abstract.replace('\n','')
            elif abstracts_type == 'json':
                with open(abstracts) as abstracts_file:
                    papers = json.load(abstracts_file)
                for pid in papers:
                    title      = papers[pid]['title']
                    title_norm = ' '.join(title.lower().split())
                    title_norm = normalize('NFC', title_norm)
                    abstract   = papers[pid]['abstract']
                    #print("**", title_norm)
                    title2abstract[title_norm] = abstract
    return title2abstract


def get_authors(authors, authors_type = 'xlsx'):
    # Function which collects the authors of a given track
    all_authors      = {}
    if authors_type == 'xlsx':
        previous_paperid = None
        author_counter   = 1
        # First, let us read the input XLSX file to turn it into some csv
        wb = load_workbook(authors, read_only=True)
        sh = wb.get_sheet_by_name('All')
        for row in sh.iter_rows(min_row=2):
            #print(row)
            if row[0].value is None:
                break
            values  = [cell.value for cell in row]  
            author  = Author()
            paperid = int(values[0])
            author.__dict__['paperid']     = str(paperid)
            if previous_paperid == paperid:
              author_counter += 1
            else:
                author_counter  = 1
            author.__dict__['rank']        = author_counter
            previous_paperid               = paperid
            if sh.max_column == 9: 
                #the excel file comes from a premium easychair account
                #the first and lastnames are in two columns
                fullname                       = HumanName(values[1] + ' ' + values[2])
                # "~" is used as an unsplittable space in first/last names
                author.__dict__['firstname']   = re.sub(r'([^\\])~', r'\1 ', fullname.first)
                author.__dict__['lastname']    = (fullname.middle + ' ' + fullname.last).strip()
                author.__dict__['email']       = values[3]
                author.__dict__['affiliation'] = html.unescape(values[5] + ', ' \
                                                               + values[4])              
            else:
                fullname                       = HumanName(values[1])
                # "~" is used as an unsplittable space in first/last names
                author.__dict__['firstname']   = re.sub(r'([^\\])~', r'\1 ', fullname.first)
                author.__dict__['lastname']    = (fullname.middle + ' ' + fullname.last).strip()
                author.__dict__['email']       = values[2]
                author.__dict__['affiliation'] = html.unescape(values[4] + ', ' \
                                                               + values[3])
            if paperid in all_authors.keys():
                all_authors[paperid].append(author)
            else:
                all_authors[paperid] = [author]
    elif authors_type == 'json':
        with open(authors) as authors_file:
            papers = json.load(authors_file)
        for pid, pauthors in papers.items():
            for prank, paut in papers[pid].items():
                author = Author()             
                author.__dict__['paperid']     = str(pid)
                author.__dict__['rank']        = prank
                author.__dict__['firstname']   = papers[pid][prank]['firstname']
                author.__dict__['lastname']    = papers[pid][prank]['lastname']
                author.__dict__['email']       = papers[pid][prank]['email']
                author.__dict__['affiliation'] = papers[pid][prank]['affiliation']
                if pid in all_authors.keys():
                    all_authors[pid].append(author)
                else:
                    all_authors[pid] = [author]
    return all_authors


def get_data(e, indir, verbose, ignore_pdf, sessions, id_col, authors_col, title_col, \
             idhal_col, file_col, accept_col, keywords_col):
    for t in e.__dict__['tracks'].keys():
        print('Processing track:', t, file=sys.stderr)
        track_dir = os.path.join(indir, t)
        if not os.path.exists(track_dir):
            print(' [Warning] Directory missing for track ' + t + \
                  ' (track ignored).' , file=sys.stderr)
            #move on to the next track
        else: 
            papers_file   = os.path.join(track_dir, 'articles.csv')
            abstracts_file= None
            authors_file  = None
            if os.path.exists(os.path.join(track_dir, 'accepted.html')):
                abstracts_file= os.path.join(track_dir, 'accepted.html')
            elif os.path.exists(os.path.join(track_dir, 'abstracts.json')):
                abstracts_file= os.path.join(track_dir, 'abstracts.json')
            else:
                if verbose > 0:
                    print('[Warning] neither accepted.html nor abstracts.json found for track ' + t + \
                          '\n(abstracts will be extracted from pdf articles)', \
                          file=sys.stderr)                
            if os.path.exists(os.path.join(track_dir, 'author_list.xlsx')):
                authors_file  = os.path.join(track_dir, 'author_list.xlsx')
            elif os.path.exists(os.path.join(track_dir, 'authors.json')):
                authors_file  = os.path.join(track_dir, 'authors.json')
            else:
                if verbose > 2:
                    print('[Warning] neither author_list.xlsx nor authors.json found for track ' + t + \
                          '\n(authors will be extracted from articles.csv and be missing affiliations)', file=sys.stderr)                
            if not os.path.exists(papers_file):
                print('\n[Error] Missing articles.csv for track ' + t + \
                      '\n(track ignored).\n', \
                      file=sys.stderr)
                continue
            accept   = 'ACCEPT' #default case
            track_obj= e.__dict__['tracks'][t]
            #if accept flag is given in the event.yml file, use it            
            if track_obj.accept != '':
                accept = track_obj.accept
            articles = get_articles(papers_file, abstracts_file, \
                                    track_dir, accept, track_obj, \
                                    verbose, ignore_pdf, sessions, \
                                    id_col, authors_col, title_col, idhal_col, file_col, \
                                    accept_col, keywords_col)
            authors  = get_authors(authors_file, os.path.splitext(authors_file)[1][1:]) if authors_file else {}
            track_obj.articles = articles
            track_obj.authors  = authors



class Track(object):

    def __init__(self, dir_name):
        self.trackid        = dir_name
        self.accept         = '' #semi-colon-separated list of acceptance keywords
        self.chairs         = '' #comma-separated list of names
        self.volume         = '' #track id (used for anthology formatting)
        self.articles       = [] #list of Article objects
        self.authors        = {} #dict associating paper ID with an ordered list of author objects
        self.partition      = {} #dict mapping session id to lists of paper ids
        self.sessions       = '' #semi-colon-separated list of session ids
        self.startpage      = -1 #for setting alternative first page number (cf reverse-engineering)
        self.order          = '' #not set by default here
        self.base_url       = '' #for TALN archive storage
        self.fullname       = '' #for track volume title

    def update_sessions(self, session, paper_id):
        if session not in self.partition:
            self.partition[session] = []
        self.partition[session].append(paper_id)
             
    def __str__(self):
        s = ''
        for t,i in self.__dict__.items(): 
            s += '\n' + t + '\n'
            for a,b in i.items():
                if type(b) == list: #articles
                    s += a + ':' + '\n  '.join(list(map(lambda x: str(x), b))) + '\n'
                elif type(b) == dict: #authors
                    flat_list = [item for sublist in b.values() for item in sublist]
                    s += a + ':' + '\n  '.join(list(map(lambda x: str(x), flat_list))) + '\n'
                else:
                    s += a + ':' + str(b) + '\n'
        return s



#!/usr/bin/python
# -*- coding: utf-8 -*-

import xml.sax

################################################################################
class content_handler(xml.sax.ContentHandler):
    """ XML sax parser for taln documents """
    #-T-----------------------------------------------------------------------T-
    def __init__(self, path):
        # Tree for pushing/poping tags, attrs
        self.tree = []
        
        # Buffer for xml element
        self.buffer = ''

        self.meta = {}
        self.articles = []
        self.info = {}
        self.current_name = {}

        self.tags = ['titre', 'title', 'resume', 'abstract', 'mots_cles',\
                     'keywords', 'type', 'acronyme', 'ville', 'pays', 'dateDebut',\
                     'dateFin', 'pages', 'siteWeb']
        
        # Construct an launch the parser
        parser = xml.sax.make_parser()
        parser.setContentHandler(self)
        parser.parse(path)
    #-B-----------------------------------------------------------------------B-
        
    #-T-----------------------------------------------------------------------T-
    def startElement(self, name, attrs):
        self.tree.append((name, attrs))

    #-B-----------------------------------------------------------------------B-
        
    #-T-----------------------------------------------------------------------T-
    def characters(self, data):
        self.buffer += data
    #-B-----------------------------------------------------------------------B-
        
    #-T-----------------------------------------------------------------------T-
    def endElement(self, name):
        tag, attrs = self.tree.pop()

        if name == 'nom':
            self.current_name['nom'] = self.buffer.strip()

        elif name == 'prenom':
            self.current_name['prenom'] = self.buffer.strip()

        elif name == 'affiliationId':
            if 'affiliationId' not in self.current_name.keys():
                # on ne garde que l'affiliation principale
                self.current_name['affiliationId'] = self.buffer.strip()

        elif name == 'affiliation':
            if not 'affiliations' in self.info.keys():
                self.info['affiliations'] = []
            self.info['affiliations'].append((attrs['affiliationId'], self.buffer.strip()))

        elif name == 'email':
            self.current_name['email'] = self.buffer.strip()
            
        # Récupère les auteurs des papiers
        elif name == 'auteur':
            if not 'auteurs' in self.info.keys():
                self.info['auteurs'] = []
            self.info['auteurs'].append( ( self.current_name['prenom'], \
                                           self.current_name['nom'], \
                                           self.current_name['affiliationId'], \
                                           self.current_name['email'] ) )

        # Récupère les noms des présidents de la conférence
        elif name == 'president':
            if not 'presidents' in self.info.keys():
                self.info['presidents'] = []
            if 'track' not in attrs:
                self.info['presidents'].append( ( self.current_name['prenom'], \
                                                  self.current_name['nom'], None ) )
            else:
                self.info['presidents'].append( ( self.current_name['prenom'], \
                                                  self.current_name['nom'], attrs['track'] ) )

        # Récupère les différents types de papiers de la conférence
        elif name == 'type' and self.tree[-1][0] == 'typeArticles':
            if not 'typeArticles' in self.info.keys():
                self.info['typeArticles'] = []
            self.info['typeArticles'].append((attrs['id'], self.buffer.strip()))

        # Récupère le titre de la conférence
        elif name == 'titre' and self.tree[-1][0] == 'edition':
            self.info['conf_titre'] = self.buffer.strip()
            
        # Récupère tous les tags terminaux
        elif name in self.tags:
            self.info[name] = self.buffer.strip()

        elif name == 'article':
            self.info['id'] = attrs['id']
            self.articles.append(self.info.copy())
            self.info = {}

        elif name == 'edition':
        	self.meta = self.info
        	self.info = {}
    
        # Flush the buffer
        self.buffer = ''
    #-B-----------------------------------------------------------------------B-

################################################################################



import os, shutil, re, sys, tqdm
import xml.etree.cElementTree as ET
from xml.dom                import minidom
from pylatexenc.latexencode import UnicodeToLatexEncoder, \
    UnicodeToLatexConversionRule, RULE_DICT #unicode_to_latex
from unidecode              import unidecode
from pybtex.database        import BibliographyData, Entry
from ..utils.archives       import volumes
from ..exports.anthology    import order_papers, order_session

BASE_URL = 'http://talnarchives.atala.org'

def texify(string, encode=False):
    """Return a modified version of the argument string where non-ASCII symbols have
    been converted into LaTeX escape codes.
    """
    u = UnicodeToLatexEncoder(
        conversion_rules=[
            UnicodeToLatexConversionRule(rule_type=RULE_DICT, rule={
                0x0251:r'\textscripta',
                0x0254:r'\textopeno',
                0x0255:r'\textctc',
                0x026A:r'\textsci',
                0x0281:r'\textinvscr',
                0x0282:r'\textrtails',
                0x0283:r'\textesh',
                0x028A:r'\textupsilon',
                0x028F:r'\textscy',
                0x0292:r'\textyogh',
                0x02B0:r'\textsuperscript{h}',
                0x02C8:r'\textsuperscript{\textpipe}',
                0x0303:r'\textsuperscript{\textasciitilde}',
                0x201F:r'\textsuperscript{\textdoublegrave{}}'
            }),
            'defaults'
        ]
    )
    s = u.unicode_to_latex(string).replace(r'{\textquoteright}',"'")\
                                  .replace(r'{\textquoteleft}',"'")\
                                  .replace(r'{\oe}',"\oe ")\
                                  .replace(r'{\textquotedblleft}', '"')\
                                  .replace(r'{\textquotedblright}', '"')\
                                  .replace(r'{\textquotedblleft}', '"')\
                                  .replace(r'{\textquotedblright}', '"')\
                                  .replace(r'{\textendash}', '-')\
                                  .replace(r'\textquotesingle', "'")\
                                  .replace(r'{\textasciitilde}','~')\
                                  .replace(r'{\textasciicircum}', "^")\
                                  .replace(r'\{', "{")\
                                  .replace(r'\}', "}")\
                                  .replace(r'\$', "$")\
                                  .replace(r'{\textbackslash}', "\\")

    if encode:
        return string
    else:
        return s

def write_xml(event, stopwords, verbose, keep, sessions, latex_encode):
    # Preparing XML content (DOM)
    root     = ET.Element('conference')
    edition  = ET.SubElement(root, 'edition')
    articles = ET.SubElement(root, 'articles')
    # Preparing output files
    eventname= event.__dict__['abbrev'] + \
               '-' + str(event.__dict__['year'])
    indir    = os.getcwd()
    xmldir   = os.path.join(indir, "out")
    xmlfile  = os.path.join(xmldir, eventname, \
                            eventname.lower() + '.xml')
    bibfile  = os.path.join(xmldir, eventname, \
                            eventname.lower() + '.bib')
    bibdir   = os.path.join(xmldir, eventname, 'bib')
    pdfdir   = os.path.join(xmldir, eventname, 'actes')
    if not os.path.exists(os.path.join(xmldir, eventname)):
        os.mkdir(os.path.join(xmldir, eventname))
    if not os.path.exists(bibdir):
        os.mkdir(bibdir)
    if not os.path.exists(pdfdir):
        os.mkdir(pdfdir)
    # Compiling conference bib content
    bibfileh = open(bibfile, 'wt')
    urlpref  = os.path.join(BASE_URL, event.__dict__['abbrev'], eventname) #default value
    if 'base_url' in event.__dict__ :
        urlpref = event.__dict__['base_url'] #the base_url is specified at the event level
    bib_data = BibliographyData(
        { event.__dict__['abbrev'] + \
          ':' + str(event.__dict__['year']) : \
          Entry('proceedings', [
              ('editor', texify(' and '.join(event.__dict__['chairs']), latex_encode)),
              ('title',  texify(event.__dict__['booktitle'], latex_encode)),
              ('month',  str(event.__dict__['month'])),
              ('year',   str(event.__dict__['year'])),
              ('address',texify(event.__dict__['location'], latex_encode)),
              ('publisher', 'Association pour le Traitement Automatique des Langues'),
              ('url',    urlpref),
          ]),
        })
    bibfileh.write(bib_data.to_string('bibtex'))
    bibfileh.write('\n\n')
    ## Process edition
    ET.SubElement(edition, 'acronyme').text = event.__dict__['abbrev'] +\
                                              "'"+str(event.__dict__['year'])
    ET.SubElement(edition, 'titre').text    = event.__dict__['title']
    ET.SubElement(edition, 'ville').text    = event.__dict__['location'].split(', ')[0]
    ET.SubElement(edition, 'pays').text     = event.__dict__['location'].split(', ')[1]
    ET.SubElement(edition, 'dateDebut').text= event.__dict__['begin'].strftime('%Y-%m-%d')
    ET.SubElement(edition, 'dateFin').text  = event.__dict__['end'].strftime('%Y-%m-%d')
    ET.SubElement(edition, 'siteWeb').text  = event.__dict__['url'] 
    presidents= ET.SubElement(edition, 'presidents')
    for c in event.__dict__['chairs']:
        president = ET.SubElement(presidents, 'president')
        ET.SubElement(president, 'prenom').text = (' '.join(c.split(' ')[:-1])).strip()
        ET.SubElement(president, 'nom').text    = c.split(' ')[-1] 
    types     = ET.SubElement(edition, 'typeArticles')
    for t in event.__dict__['tracks']:
        ET.SubElement(types, 'type', id=t).text = dict(volumes)[t] if t in dict(volumes) else event.__dict__['tracks'][t].__dict__['fullname'].replace('\\','')
    ## Process articles
    for t in event.__dict__['tracks']:
        print('Processing track:', t, file=sys.stderr)
        order = 'input' #default
        start = 1       #default
        ## Logging (keep submissions info)
        if keep:
            logfile = open(os.path.join(indir,'submissions-'+eventname+'-'+t+'.in.csv'),'w') 
        papers  = event.__dict__['tracks'][t].articles
        # track's config overrides command line one
        if event.__dict__['tracks'][t].order != '':
            order = event.__dict__['tracks'][t].order
        ordered = papers
        if sessions and event.__dict__['tracks'][t].sessions != '':
            ordered = order_session(papers, order, stopwords, verbose, \
                                    event.__dict__['tracks'][t])
        else:
            ordered = order_papers(papers, order, stopwords, verbose, \
                                   event.__dict__['tracks'][t].authors)
        # Track settings overwrite cli start option
        if int(event.__dict__['tracks'][t].startpage) > 0:
            start = int(event.__dict__['tracks'][t].startpage)
        current_pos = start
        for art in tqdm.tqdm(ordered):
            if art:
                if keep:
                    LOG_PATTERN = ['paperid', 'authors', 'title', 'track', 'url', \
                               'title2', 'language', 'accept', 'keywords']
                    s = []
                    for y in LOG_PATTERN:
                        if y == 'url':
                            s.append(os.path.basename(art.__dict__[y]))
                        elif y == 'accept':
                            s.append('ACCEPT')
                        else:                            
                            if type(art.__dict__[y]) == str :
                                s.append(art.__dict__[y])
                            else: 
                                s.append(', '.join(art.__dict__[y]))
                    print('\t'.join(s),file=logfile) 
                ## Updating XML file
                article = ET.SubElement(articles, 'article', id=art.__dict__['paperid'])
                auteurs = ET.SubElement(article, 'auteurs')
                affilia = []
                autbib  = [] #authors as bibtex strings
                try:
                    for au in event.__dict__['tracks'][t].authors[int(art.__dict__['paperid'])]:
                        auteur  = ET.SubElement(auteurs, 'auteur')
                        autbib.append((au.__dict__['firstname'], \
                                       au.__dict__['lastname']))
                        ET.SubElement(auteur, 'prenom').text       = au.__dict__['firstname']
                        ET.SubElement(auteur, 'nom').text          = au.__dict__['lastname']
                        ET.SubElement(auteur, 'email').text        = au.__dict__['email']
                        ET.SubElement(auteur, 'affiliationId').text= str(au.__dict__['rank'])
                        affilia.append(au.__dict__['affiliation'])
                except KeyError:
                    for aut2 in art.__dict__['authors'].replace(' and ', ', ').split(', '):
                        aut     = re.sub(r'([^\\])~', r'\1 ', aut2)
                        auteur  = ET.SubElement(auteurs, 'auteur')
                        fname   = aut.split(' ')[0]
                        lname   = ' '.join(aut.split(' ')[1:])
                        autbib.append((fname, lname))
                        ET.SubElement(auteur, 'prenom').text       = fname
                        ET.SubElement(auteur, 'nom').text          = lname
                affiliations = ET.SubElement(article, 'affiliations')
                for aff in range(len(affilia)):
                    ET.SubElement(affiliations, 'affiliation', \
                                  affiliationId=str(aff+1)).text = affilia[aff]
                if art.__dict__['language'] == 'fr':
                    ET.SubElement(article, 'titre').text    = art.__dict__['title']
                    ET.SubElement(article, 'resume').text   = art.__dict__['abstract']
                    ET.SubElement(article, 'mots_cles').text= art.__dict__['keywords']
                    ET.SubElement(article, 'title').text    = ''
                    ET.SubElement(article, 'abstract').text = ''
                    ET.SubElement(article, 'keywords').text = ''
                    ET.SubElement(article, 'language').text = 'french' 
                else:
                    ET.SubElement(article, 'titre').text    = ''
                    ET.SubElement(article, 'resume').text   = ''
                    ET.SubElement(article, 'mots_cles').text= ''
                    ET.SubElement(article, 'title').text    = art.__dict__['title']
                    ET.SubElement(article, 'abstract').text = art.__dict__['abstract']
                    ET.SubElement(article, 'keywords').text = art.__dict__['keywords']
                    ET.SubElement(article, 'language').text = 'english' 
                ET.SubElement(article, 'type').text         = art.__dict__['track']
                pages = str(current_pos) + '-' + str(current_pos + art.__dict__['numpages']-1)
                current_pos += art.__dict__['numpages']
                ET.SubElement(article, 'pages').text        = pages
                ## Generate article bib file (and populating event bib file)
                if 'base_url' in event.__dict__['tracks'][t].__dict__: #urlpref is defined at the track level
                    urlpref = event.__dict__['tracks'][t].__dict__['base_url']
                bib_art = BibliographyData({
                    '-'.join(list(map(lambda x: unidecode(x[1].replace(' ', '-')), autbib)))+':'+\
                    event.__dict__['abbrev'] + ':' + str(event.__dict__['year']) \
                    : Entry('inproceedings', [
                        ('author',    texify(' and '.join(list(map(lambda x: x[1] + ', ' + x[0], \
                                                                   autbib))), latex_encode)),
                        ('title',     texify(art.__dict__['title'], latex_encode)),
                        ('booktitle', texify(event.__dict__['booktitle'] + '. ' + \
                                             event.__dict__['tracks'][t].fullname, latex_encode)),
                        ('month',     str(event.__dict__['month'])),
                        ('year',      str(event.__dict__['year'])),
                        ('address',   texify(event.__dict__['location'], latex_encode)),
                        ('publisher', 'Association pour le Traitement Automatique des Langues'),
                        ('pages',     pages),
                        ('note',      texify(art.__dict__['title2'], latex_encode)), 
                        ('abstract',  texify(art.__dict__['abstract'], latex_encode)),
                        ('keywords',  texify(art.__dict__['keywords'], latex_encode)),
                        ('url',       os.path.join(urlpref, art.__dict__['paperid']+'.pdf')),
                    ]),
                })
                bibfileh.write(bib_art.to_string('bibtex'))
                bibfileh.write('\n\n')
                bib_art.to_file(os.path.join(bibdir, art.__dict__['paperid'] + '.bib'), 'bibtex')
                ## Copy pdf file
                if os.path.exists(os.path.join(pdfdir, art.__dict__['paperid'] + '.pdf')):
                    os.remove(os.path.join(pdfdir, art.__dict__['paperid'] + '.pdf'))
                pdf_indir = os.path.join(indir, t, 'pdf')
                if os.path.exists(os.path.join(indir, 'out', 'pdf_'+eventname, t, 'pdf_out')):
                    pdf_indir = os.path.join(indir, 'out', 'pdf_'+eventname, t, 'pdf_out')
                st = os.stat(os.path.join(pdf_indir, os.path.basename(art.__dict__['url'])))
                shutil.copy(os.path.join(pdf_indir, os.path.basename(art.__dict__['url'])), \
                            os.path.join(pdfdir, art.__dict__['paperid'] + '.pdf'))
                os.chmod(os.path.join(pdfdir, art.__dict__['paperid'] + '.pdf'), st.st_mode)
            else:
                current_pos += 1
        ## Close logging if needed
        if keep:
            logfile.close()
    ## Close file handler and write down XML content
    bibfileh.close()
    if verbose > 2:
        xmls   = ET.tostring(root, encoding='utf-8', short_empty_elements=False).decode()
        print(xmls)
    xmlstr = minidom.parseString(ET.tostring(root, encoding='utf-8', 
                                             short_empty_elements=False)).\
                                             toprettyxml(indent='   ')
    with open(xmlfile, 'w') as f:
        f.write(xmlstr)



import os, sys, shutil, subprocess, json, tqdm
from urllib.parse          import quote
from pylatexenc.latex2text import LatexNodes2Text
from ..exports.anthology   import order_papers, order_session

def get_id(event, indir, stopwords, verbose, sessions):
    for t in event.__dict__['tracks']:
        print('Processing track:', t, file=sys.stderr)
        idfile = open(os.path.join(indir, 'idhal-' + event.__dict__['abbrev'] + '-' + \
                                   str(event.__dict__['year']) + '-' + t +'.csv'),'w')
        papers  = event.__dict__['tracks'][t]['articles']
        order = 'input'
        if 'order' in event.__dict__['tracks'][t].keys():
            order = event.__dict__['tracks'][t]['order']
        ordered     = papers
        if sessions and event.__dict__['tracks'][t].sessions != '':
            ordered = order_session(papers, order, stopwords, verbose, \
                                    event.__dict__['tracks'][t])
        else:
            ordered = order_papers(papers, order, stopwords, verbose, \
                                   event.__dict__['tracks'][t]['authors'])
        for art in tqdm.tqdm(ordered):
            if art:
                get_id_article(art, idfile, verbose)
        idfile.close()

def get_id_article(art, idfile, verbose):
    command = ["curl", "-s", \
               'https://api.archives-ouvertes.fr/search/?q=title_s:\"'\
               + quote(LatexNodes2Text().latex_to_text(art.__dict__['title'])).\
               replace('%C2%A0', '%20').replace('%20%20%20', '%26')\
               + '\"&fl=halId_s']
    out = subprocess.check_output(command)
    nb_id = int(json.loads(out.decode())['response']['numFound'])
    if nb_id > 0:
        hal = ';'.join(set([json.loads(out.decode())['response']['docs'][i]['halId_s'] \
                        for i in range(nb_id)]))
        s   = [art.__dict__['paperid'], art.__dict__['title'], hal] 
        print('\t'.join(s),file=idfile)
        if nb_id > 1 and verbose > 1:
            print('[Info] Several HAL identifiers found for paper ' + art.__dict__['paperid'], \
              file=sys.stderr)
    else:
        if verbose > 1:
            print('[Info] HAL identifier not found for paper ' + art.__dict__['paperid'], \
                  file=sys.stderr)
            print('[Info] command used:\n' + ' '.join(command), file=sys.stderr)
            print('[Info] paper title :\n' + art.__dict__['title'], file=sys.stderr)
        s   = [art.__dict__['paperid'], art.__dict__['title'], 'NOT AVAILABLE']
        print('\t'.join(s),file=idfile)



import os, sys, shutil, re, subprocess, pypdf
from   langdetect  import detect

def read_pdf(article, indir, path, default_title, ignore_pdf, verbose):
    pdf      = pypdf.PdfReader(os.path.join(indir, 'pdf', path), \
                                strict=False)
    numpages = pdf.trailer['/Root']['/Pages']['/Count']
    #print(article.__dict__['paperid'], numpages,'pages')
    article.__dict__['numpages'] = numpages
    try:
        if shutil.which('pdftotext') is None:
            print('[Error] pdftotext is missing. ' + \
                  'Please install the xPdf command line tools,' + \
                  '(see http://www.xpdfreader.com/pdftotext-man.html).', \
                  file=sys.stderr)
            sys.exit(199)
        args = ["pdftotext",
                '-nopgbrk',
                os.path.join(indir, 'pdf', path),
                '-']
        res = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        paper_content = res.stdout.decode('utf-8')
        #print('*',paper_content)
        m = None
        if article.__dict__['language'] == 'fr':
            #Searching French Abstract:
            mabs = re.search('[ÉE]\s*S\s*U\s*M\s*[ÉE][\_\.]*\n*(.+?)\n+' + \
                             'A\s*B\s*S\s*T\s*R\s*A\s*C\s*T', \
                             paper_content, re.DOTALL)
            #Search French keywords:
            mkey = re.search('M\s*O\s*T\s*S\s*-\s*C\s*L\s*[ÉE]\s*S\s*:\s*(.+?)\n+' + \
                             'K\s*E\s*Y\s*', \
                             paper_content, re.IGNORECASE|re.DOTALL)
            #Searching English Title:
            m = re.search('B\s*S\s*T\s*R\s*A\s*C\s*T\s*\_*\n+([^\.]+?)\.', \
                          paper_content, re.DOTALL)
        else:
            #Searching Englich Abstract:
            mabs = re.search('A\s*B\s*S\s*T\s*R\s*A\s*C\s*T\s*[\_\.]*\n*(.+?)\n+' + \
                             '(M\s*O\s*T\s*S|R\s*[ÉE]\s*S\s*U)', \
                             paper_content, re.IGNORECASE|re.DOTALL)
            #Search English keywords:
            mkey = re.search('K\s*E\s*Y\s*W\s*O\s*R\s*D\s*S\s*:\s*(.+?)\n+' + \
                             '1\s*', \
                             paper_content, re.IGNORECASE|re.DOTALL)
            #Searching French Title:
            m = re.search('[ÉE]\s*S\s*U\s*M\s*[ÉE]\n+([^\.]+?)\.', \
                          paper_content, re.IGNORECASE|re.DOTALL)
            if m: #secondary title found (right after RÉSUMÉ / ABSTRACT)
                #in case the title is split over lines, we keep lines which
                #do not start with an upper case letter (except when the preceding 
                #line ends with more than 3 letters, e.g. it is not the determiner "A") 
                title2 = re.sub(r'([\w-]{4,})(\s*[\?”"]?)\s*\n+[A-Z]', r'\1\2~', \
                                m.group(1)).split('~')[0].replace('\n', ' ')
                #print('\n*** before:', article.__dict__['paperid'], m.group(1), '***\n')
                #print('\n*** found:', article.__dict__['paperid'], title2, '***\n')
                article.__dict__['title2']    = title2
                article.__dict__['language2'] = detect(title2)
            else: # for one paper, unicode was needed
                if verbose > 2:
                    print('[Warning] Secondary title extraction failed for paper ', \
                          article.__dict__['title'])
                        #print(paper_content)
            if mabs and (not ignore_pdf or article.__dict__['abstract'] == ''):
                article.__dict__['abstract'] = mabs.group(1).replace('\n', ' ')
                #print('*',article.__dict__['abstract'])
            if mkey and (not ignore_pdf or article.__dict__['keywords'] == ''):
                article.__dict__['keywords'] = mkey.group(1).replace('\n',' ')
                #print('*',article.__dict__['keywords'])
    except UnicodeDecodeError as ud:
        if verbose >= 0:
            print('[Warning] encoding error, ' + \
                  'paper ignored for pdf title/abstract extraction:', \
                  article.__dict__['title'], '('+article.__dict__['paperid']+')', \
                  file=sys.stderr)
        #English title read in column 4 in csv file (fallback)
        article.__dict__['title2'] = default_title



from unidecode   import unidecode

class Author(object): 

    def __init__(self):
        self.firstname   = ''
        self.lastname    = ''
        self.affiliation = ''
        self.paperid     = ''
        self.rank        = 0  #position among authors
        self.email       = ''

    def __name4ordering__(self): #method used to compute a sortable full name
        return (unidecode(self.lastname) + unidecode(self.firstname)).replace('~','')

    def __str__(self):
        s = ''
        for k,v in self.__dict__.items():
            s += k + ':' + str(v) + '\n'
        return s



import os, sys, re, shutil, subprocess, tqdm, pypdf, stat, json
from distutils             import dir_util
from urllib.parse          import quote
from pylatexenc.latex2text import LatexNodes2Text
from ..exports.anthology   import order_papers, order_session
from ..exports.archives    import texify

#months_fr = ['janvier', 'février', 'mars', 'avril', 'mai', 'juin', \
#             'juillet', 'août', 'septembre', 'octobre', 'novembre', 'décembre']

def clean_tex(f):
    if os.path.exists(f):
        os.unlink(f)


def compile_tex(orig, dest, tex_template, tex_log):
    os.chdir(dest)    
    clean_tex(tex_template[:-4]+'.log')
    clean_tex(tex_template[:-4]+'.aux')
    clean_tex(tex_template[:-4]+'.out')
    clean_tex(tex_template[:-4]+'.toc')
    clean_tex('all_papers.aux')    
    subprocess.call(['pdflatex', tex_template], \
                    stdout=subprocess.DEVNULL, 
                    stderr=subprocess.DEVNULL, \
                    timeout=30)
    subprocess.call(['pdflatex', tex_template], \
                    stdout=subprocess.DEVNULL, \
                    stderr=subprocess.DEVNULL, \
                    timeout=30)
    if not tex_log:
        clean_tex(tex_template[:-4]+'.log')
        clean_tex(tex_template[:-4]+'.aux')
        clean_tex(tex_template[:-4]+'.out')
        clean_tex(tex_template[:-4]+'.toc')
        clean_tex('all_papers.aux')    
    os.chdir(orig)

    
def getBookmarkPageNumber(filename):
    def review_bookmarks(pdf, bookmarks, label):
        res = -1
        for b in bookmarks:
            if type(b) == list:
                res = max(res, review_bookmarks(pdf, b, label))
            else:
                if b.title == label:
                    res = pdf.get_destination_page_number(b) + 1 #page count starts from 0
        return res
    with open(filename, "rb") as f:
       pdf = pypdf.PdfReader(f)
       return review_bookmarks(pdf, pdf.outline, 'EndofFrontmatter')


def updateTemplateString(file_in, chairs, event, t, exists, halid, sessions, start, paper_pdf, paper_title, paper_authors):
    # to replace: title - abbr - year - month - chairs - 
    # location - publisher - booktitle
    text_out = []
    for line_in in file_in:
        line_out = line_in 
        for check in ['background', 'logo', 'sponsors', 'credits', \
                      'message', 'preface', 'committee', 'invited', 'isbn']:
            if check in exists.keys():
                line_out = line_out.replace(check.upper()+'-PLACEHOLDER', \
                                            '\\'+check+'true')
            else:
                line_out = line_out.replace(check.upper()+'-PLACEHOLDER', \
                                            '\\'+check+'false')
        if halid:
            line_out = line_out.replace('HAL-PLACEHOLDER', \
                                        '\\HAL{\\halid}')
        else:
            line_out = line_out.replace('HAL-PLACEHOLDER', \
                                        '')
        if sessions:
            line_out = line_out.replace('SESSION-PLACEHOLDER', \
                                        '\\cursession,')
        else:
            line_out = line_out.replace('SESSION-PLACEHOLDER', \
                                        '')

        line_out = line_out.replace('TRACK-NAME-PLACEHOLDER', \
                                    event.__dict__['tracks'][t].fullname)
        line_out = line_out.replace('PAGE-PLACEHOLDER',     \
                                    str(start))
        line_out = line_out.replace('PAPER-PDF',     \
                                    paper_pdf)
        line_out = line_out.replace('PAPER-TITLE',     \
                                    paper_title)
        line_out = line_out.replace('PAPER-AUTHORS',     \
                                    paper_authors)
        line_out = line_out.replace('TITLE-PLACEHOLDER',     \
                                    event.__dict__['title'].\
                                    replace(' ; ','\\\\%\n'))
        line_out = line_out.replace('ABBREV-PLACEHOLDER',    \
                                    event.__dict__['abbrev'])
        line_out = line_out.replace('SHORT-PLACEHOLDER',    \
                                    event.__dict__['shortbooktitle'])
        line_out = line_out.replace('YEAR-PLACEHOLDER',    \
                                    str(event.__dict__['year']))
        begin = event.__dict__['begin'].strftime('%Y-%m-%d')
        end   = event.__dict__['end'].strftime('%Y-%m-%d')
        line_out = line_out.replace('DATE-PLACEHOLDER',      \
                                    '{\\origdate \\daterange{' + \
                                    begin + '}{' + end +'} }')
        line_out = line_out.replace('URL-PLACEHOLDER',      \
                                    str(event.__dict__['url']))
        line_out = line_out.replace('CHAIRS-PLACEHOLDER',     \
                                    chairs)
        line_out = line_out.replace('LOCATION-PLACEHOLDER',  \
                                    event.__dict__['location'])
        line_out = line_out.replace('PUBLISHER-PLACEHOLDER',  \
                                    event.__dict__['publisher'])
        line_out = line_out.replace('BOOK-PLACEHOLDER',      \
                                    event.__dict__['texbooktitle'] \
                                    if 'texbooktitle' in event.__dict__ \
                                    else event.__dict__['booktitle'])
        text_out.append(line_out)
    return ''.join(text_out)


def write_tex(event, stopwords, verbose, halid, sessions, tex_log, img, xlang):
    indir    = os.getcwd()
    outdir   = os.path.join(indir, "out")
    # Preparing output files
    eventname= event.__dict__['abbrev'] + \
               '-' + str(event.__dict__['year'])    
    if not os.path.exists(os.path.join(outdir, 'pdf_' + eventname)):
        os.mkdir(os.path.join(outdir, 'pdf_' + eventname))
    ## Process articles
    for t in event.__dict__['tracks']:
        print('Processing track:', t, file=sys.stderr)
        chairs = ', '.join(list(map(lambda x: x.split(', ')[1] + \
                                    ' ' + x.split(', ')[0], \
                                    event.__dict__['chairs'])))
        if event.__dict__['tracks'][t].chairs != '':
            chairs = event.__dict__['tracks'][t].chairs                
        source = indir
        dest   = os.path.join(outdir, 'pdf_' + eventname, t)
        if not os.path.exists(dest):
            os.mkdir(dest)
        if not os.path.exists(os.path.join(dest, 'pdf')):
            os.mkdir(os.path.join(dest, 'pdf'))
        if not os.path.exists(os.path.join(dest, 'pdf_out')):
            os.mkdir(os.path.join(dest, 'pdf_out'))
        allpapers = open(os.path.join(dest, 'all_papers.tex'), 'wt')
        ## Process paper in right order (cf automatic numbering)
        papers  = event.__dict__['tracks'][t].articles
        # track's config overrides command line one
        start = 1 #default
        if int(event.__dict__['tracks'][t].startpage) > 0:
            start = int(event.__dict__['tracks'][t].startpage)
        order = 'input' #default
        if event.__dict__['tracks'][t].order != '':
            order = event.__dict__['tracks'][t].order
        ordered     = papers
        if sessions and event.__dict__['tracks'][t].sessions != '':
            ordered = order_session(papers, order, stopwords, verbose, \
                                    event.__dict__['tracks'][t])
        else:
            ordered = order_papers(papers, order, stopwords, verbose, \
                                   event.__dict__['tracks'][t].authors)
        for art in tqdm.tqdm(ordered):
            if art:
                process_article(event, t, indir, os.path.join(outdir, 'pdf_' + eventname), \
                                art, allpapers, halid, sessions)
            #no else since session's separator pages (i.e. when art is None) are taken 
            #care of by latex
        allpapers.close()
        ## Compiling PDF proceedings
        ## Copy files
        TEXSOURCES  =['pre-proceedings.tex', 'pre-proceedings-en.tex', 'single_paper.tex', 'by.eps']
        EVENTSOURCES=['background.png', 'logo.png','sponsors.png', 'credits.tex']
        TRACKSOURCES=['message.tex', 'preface.tex', 'committee.tex', 'invited.tex', 'isbn.tex']
        exists      = {}
        for x in TEXSOURCES:
            shutil.copy(os.path.join(source,x), dest)
        for y in EVENTSOURCES: 
            if os.path.exists(os.path.join(source, y)):
                shutil.copy(os.path.join(source, y), dest)
                filename = y.split('.')[0]
                exists[filename] = True
            else:
                print('[Info] file ' + y + ' not found', \
                      file=sys.stderr)
        for z in TRACKSOURCES:
            if os.path.exists(os.path.join(source, t, z)):
                shutil.copy(os.path.join(source, t, z), dest)
                filename = z.split('.')[0]
                exists[filename] = True
            else:
                print('[Info] file ' + z + ' not found in ' + t, \
                      file=sys.stderr)
        ## Copy users' images if needed
        if img is not None and img != "":
            dir_util.copy_tree(img, dest)
        ## Prepare proceedings
        if xlang == 'fr':
            tex_input = 'pre-proceedings.tex'
        elif xlang == 'en':
            tex_input = 'pre-proceedings-en.tex'
        else:
            print('[Warning] Unknown language for pdf output, default one (fr) used.', file=sys.stderr)
        with open(os.path.join(dest, tex_input), 'rt') as file_in:
            text_out = updateTemplateString(file_in, chairs, event, t, exists, halid, sessions, \
                                            start, '', '', '')
            with open(os.path.join(dest, 'proceedings.tex'), 'wt') as file_out:            
                file_out.write(text_out)
        #Let us now compile the proceedings
        curdir = os.getcwd()
        try:
            compile_tex(curdir, dest, 'proceedings.tex', tex_log) ## actual compilation       
            # let us finally extract frontmatter and papers
            full_proc  = os.path.join(outdir, 'pdf_'+eventname,t, 'proceedings.pdf')
            pdfreader  = pypdf.PdfReader(full_proc, strict=False)
            endoffront = getBookmarkPageNumber(full_proc)
            if ordered and not(ordered[0]): #session
                endoffront += 1 
            frontprint = pypdf.PdfWriter() 
            for cpt in range(endoffront-1):
                PageObj = pdfreader.pages[cpt]
                cpt += 1
                frontprint.add_page(PageObj)
            frontout= open(os.path.join(outdir, 'pdf_' + eventname, t, 'frontmatter.pdf'), \
                           'wb')
            frontprint.write(frontout)
            frontout.close()
            # let us then compute papers' pages (to update their header/footer)
            print('Compiling pdfs of articles ...')
            shift = start
            for art in ordered:
                if art:
                    fname  = os.path.basename(art.__dict__['url'])[:-4]+'.tex'
                    fpath  = os.path.join(outdir, 'pdf_' + eventname, t, 'pdf_out', fname)
                    shutil.copy(os.path.join(source,'single_paper.tex'), fpath)
                    shutil.copy(os.path.join(source,'by.eps'), os.path.join(outdir, 'pdf_' + eventname, t, 'pdf_out', 'by.eps'))
                    with open(os.path.join(source,'single_paper.tex'), 'rt') as f_in:
                        t_out = updateTemplateString(f_in, chairs, event, t, exists, halid, sessions, \
                                                     shift, os.path.join('..','pdf',fname[:-4]+'.pdf'),\
                                                     art.__dict__['title'], art.__dict__['authors'])
                        with open(fpath, 'wt') as f_out:
                            f_out.write(t_out)
                    compile_tex(curdir, os.path.join(outdir, 'pdf_' + eventname, t, 'pdf_out'), \
                                fname, tex_log)
                    shift += art.__dict__['numpages']
                #we skip session's delimiter
                else:
                    shift += 1
                    
        except subprocess.TimeoutExpired as toe:
            print('\n!!!! LaTeX compilation failed !!!!\n', file=sys.stderr)
            #sys.exit(198)
            continue

def process_article(event, track, idir, odir, art, allpapersh, halid, sessions):
    origfile= os.path.join(idir, track, 'pdf', os.path.basename(art.__dict__['url']))
    outfile = os.path.join(odir, track, 'pdf', os.path.basename(art.__dict__['url']))
    shutil.copy(origfile, outfile)
    os.chmod(outfile, stat.S_IRUSR | stat.S_IWUSR) #set read/write rights
    ## Extract hyperlinks using pax/PDFBox
    if shutil.which('pdfannotextractor') is None:
        print('Error: pax tex package is missing ' + \
              '(https://www.ctan.org/tex-archive/macros/latex/contrib/pax), ' + \
              'in deb-based linux, it is part of texlive-latex-extra.', file=sys.stderr)
        sys.exit(199)
    else:
        os.environ["CLASSPATH"] = os.path.join('tex','PDFBox-0.7.3.jar')
        try:
            subprocess.call(['pdfannotextractor',outfile], \
                            stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, timeout=20)
        except subprocess.TimeoutExpired as toe:
            print('Hyperlink extraction failed', file=sys.stderr)
    ## Update allpapers.tex
    hal = ''
    if halid:
        if art.__dict__['idhal']=='':
            out = subprocess.check_output(["curl", "-s", \
                                           'https://api.archives-ouvertes.fr/search/?q=title_s:\"'\
                               + quote(LatexNodes2Text().latex_to_text(art.__dict__['title'])).replace('%C2%A0', '%20')\
                                           .replace('%20%20%20', '%26') + '\"&fl=halId_s'])
            if int(json.loads(out.decode())['response']['numFound']) == 1:
                hal = json.loads(out.decode())['response']['docs'][0]['halId_s']
            elif int(json.loads(out.decode())['response']['numFound']) > 1:
                hal = json.loads(out.decode())['response']['docs'][0]['halId_s']
                print('[Warning] Several HAL identifiers found for paper ' + art.__dict__['paperid'] \
                      + '. Using ' + hal, \
                      file=sys.stderr)
            else:
                print('[Warning] HAL identifier not found for paper ' + art.__dict__['paperid'], \
                      file=sys.stderr)
        else:
            hal = art.__dict__['idhal']
            print('[Info] Using default id-hal from the csv file for paper ' + art.__dict__['paperid'] \
                  + ' (' + hal + ')', \
                  file=sys.stderr)
    session = ''
    if sessions and art.__dict__['session'] != '':
        session = art.__dict__['session']
    s = '\\goodpaper{pdf/' + os.path.basename(art.__dict__['url']) + '}{' \
        + texify(art.__dict__['title']) + '}%\n{' + \
        texify(art.__dict__['authors'].replace(' and ', ', ')) + \
        '}{'+hal+'}{'+session+'}\n\n' 
    allpapersh.write(s)



#!/usr/bin/env python3

#
# easy2acl.py - Convert data from EasyChair for use with ACLPUB
#
# Original Author: Nils Blomqvist
# Forked/modified by: Asad Sayeed
# Further modifications and docs (for 2019 Anthology): Matt Post
# Index for LaTeX book proceedings: Mehdi Ghanimifard and Simon Dobnik
#
# Please see the documentation in the README file at http://github.com/acl-org/easy2acl.

import os
import re
import sys

from csv import DictReader
from glob import glob
from shutil import copy, rmtree
#from unicode_tex import unicode_to_tex
from pylatexenc.latexencode import unicode_to_latex
from nameparser import HumanName
from pybtex.database import BibliographyData, Entry
from pypdf import PdfReader

## added by YP
def strtobool (val):
    """Convert a string representation of truth to true (1) or false (0).
    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return 1
    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
        return 0
    else:
        raise ValueError("invalid truth value %r" % (val,))

volume_start_page = 1
latex_encode      = False
if len(sys.argv) > 1:
    try:
        volume_start_page = int(sys.argv[1])
    except ValueError:
        pass
if len(sys.argv) > 2:
    try:
        latex_encode = strtobool(sys.argv[2])
    except ValueError:
        pass
##---


def texify(string, encode=latex_encode):
    """Return a modified version of the argument string where non-ASCII symbols have
    been converted into LaTeX escape codes.

    """
    #return ' '.join(map(unicode_to_tex, string.split())).replace(r'\textquotesingle', "'").replace(r'\%', '%')
    s = unicode_to_latex(string, replacement_latex_protection='braces-all').replace(r'{\textquoteright}',"'")
    if encode:
        return s
    else:
        return string.replace('_','\_')

#,----
#| Metadata
#`----
metadata = { 'chairs': [] }
with open('meta') as metadata_file:
    for line in metadata_file:
        key, value = line.rstrip().split(maxsplit=1)
        if key == 'chairs':
            metadata[key].append(value)
        else:
            metadata[key] = value

for key in 'abbrev volume title shortbooktitle booktitle month year location publisher chairs'.split():
    if key not in metadata:
        print('Fatal: missing key "{}" from "meta" file'.format(key))
        sys.exit(1)

venue = metadata["abbrev"]
volume = metadata["volume"]
year = metadata["year"]

#
# Build a dictionary of submissions (which has author information).
#
submissions = {}

with open('submissions') as submissions_file:
    for line in submissions_file:
        entry = line.rstrip().split("\t")
        submission_id = entry[0]
        authors = entry[1].replace(' and', ',').split(', ')
        title = entry[2]

        submissions[submission_id] = (title, authors)
    print("Found ", len(submissions), " submitted files")

#
# Append each accepted submission, as a tuple, to the 'accepted' list.
# Order in this file is used to determine program order.
#
accepted = []

with open('accepted') as accepted_file:
    for line in accepted_file:
        entry = line.rstrip().split("\t")
        # modified here to filter out the rejected files rather than doing
        # that by hand
        if entry[-1] == 'ACCEPT':
            submission_id = entry[0]
            title = entry[1]
            authors = submissions[submission_id][1] if submission_id != '-1' else []

            accepted.append((submission_id, title, authors))
    print("Found ", len(accepted), " accepted files")

# Read abstracts
abstracts = {}
if os.path.exists('submission.csv'):
    with open('submission.csv') as csv_file:
        d = DictReader(csv_file)
        for row in d:
            abstracts[row['#']] = row['abstract']
    print('Found ', len(abstracts), 'abstracts')
else:
    print('No abstracts available.')

#
# Find all relevant PDFs
#

# The PDF of the full proceedings
full_pdf_file = f'pdf/{venue}_{year}.pdf'.format(venue, year)
if not os.path.exists(full_pdf_file):
    print("Fatal: could not find full volume PDF '{}'".format(full_pdf_file))
    sys.exit(1)

# The PDF of the frontmatter
frontmatter_pdf_file = f'pdf/{venue}_{year}_frontmatter.pdf'.format(venue, year)
if not os.path.exists(frontmatter_pdf_file):
    print("Fatal: could not find frontmatter PDF file '{}'".format(frontmatter_pdf_file))
    sys.exit(1)

# File locations of all PDFs (seeded with PDF for frontmatter)
pdfs = { '0': frontmatter_pdf_file }
for pdf_file in glob(f'pdf/{venue}_{year}_paper_*.pdf'.format(venue, year)):
    submission_id = pdf_file.split('_')[-1].replace('.pdf', '')
    pdfs[submission_id] = pdf_file

# List of accepted papers (seeded with frontmatter)
accepted.insert(0, ('0', metadata['booktitle'], metadata['chairs']))

#
# Create Anthology tarball
#

# Create destination directories
for dir in ['bib', 'pdf']:
    dest_dir = os.path.join('proceedings/cdrom', dir)
    if not os.path.exists(dest_dir):
        os.makedirs(dest_dir)

# Copy over "meta" file
print('COPYING meta -> proceedings/meta', file=sys.stderr)
copy('meta', 'proceedings/meta')

final_bibs = []
start_page = volume_start_page #1
paper_id   = 0
for entry in accepted:
    submission_id, paper_title, authors = entry
    ## added by YP (paper_id -1 means session's delimiter, ignored for bibtex/pdf export)
    if submission_id == '-1':
        start_page += 1
        continue # move on to the next paper
    ##
    auts = []
    for aut in authors:
        name = HumanName(aut)
        # "~" is used an unsplittable space in first/last names
        new  = re.sub(r'([^\\])~', r'\1 ', name.surnames) + ', ' + \
               re.sub(r'([^\\])~', r'\1 ', name.first)
        auts.append(new)
    #authors = ' and '.join(authors)
    authors = ' and '.join(auts)
    if not submission_id in pdfs:
        print('Fatal: no PDF found for paper', paper_id, file=sys.stderr)
        sys.exit(1)

    pdf_path = pdfs[submission_id]
    dest_path = f'proceedings/cdrom/pdf/{year}.{venue}-{volume}.{paper_id}.pdf'.format(year, venue, volume, paper_id)

    copy(pdf_path, dest_path)
    print('COPYING', pdf_path, '->', dest_path, file=sys.stderr)

    bib_path = dest_path.replace('pdf', 'bib')
    if not os.path.exists(os.path.dirname(bib_path)):
        os.makedirs(os.path.dirname(bib_path))

    anthology_id = os.path.basename(dest_path).replace('.pdf', '')

    bib_type = 'inproceedings' if submission_id != '0' else 'proceedings'
    language = []
    if paper_title.endswith('[In French]'):
        paper_title = paper_title[:-len(' [In French]')]
        language    = [('language', 'French')]
    bib_entry = Entry(bib_type, [
        ('author', texify(authors)),
        ('title', texify(paper_title)),
        ('year', metadata['year']),
        ('month', metadata['month']),
        ('address', texify(metadata['location'])),
        ('publisher', texify(metadata['publisher'])),
    ] + language)

    # Add page range if not frontmatter
    if paper_id > 0:
        with open(pdf_path, 'rb') as in_:
            reader    = PdfReader(in_)
            last_page = start_page + len(reader.pages) - 1
            bib_entry.fields['pages'] = '{}--{}'.format(start_page, last_page)
            start_page = last_page + 1

    # Add the abstract if present
    if submission_id in abstracts:
        bib_entry.fields['abstract'] = texify(abstracts.get(submission_id))

    # Add booktitle for non-proceedings entries
    if bib_type == 'inproceedings':
        bib_entry.fields['booktitle'] = texify(metadata['booktitle'])

    try:
        bib_string = BibliographyData({ anthology_id: bib_entry }).to_string('bibtex')
    except TypeError as e:
        print('Fatal: Error in BibTeX-encoding paper', submission_id, file=sys.stderr)
        sys.exit(1)
    final_bibs.append(bib_string)
    with open(bib_path, 'w') as out_bib:
        print(bib_string, file=out_bib)
        print('CREATED', bib_path)

    # Increment paper_id
    paper_id += 1

# Create an index for LaTeX book proceedings
if not os.path.exists('book-proceedings'):
    os.makedirs('book-proceedings')

with open('book-proceedings/all_papers.tex', 'w') as book_file:
    for entry in accepted:
        submission_id, paper_title, authors = entry
        if paper_title.endswith('[In French]'):
            paper_title = paper_title[:-len(' [In French]')]
        if submission_id == '0' or submission_id == '-1':
            continue
        if len(authors) > 1:
            authors = ', '.join(authors[:-1]) + ' and ' + authors[-1]
        else:
            authors = authors[0]

        print("""\goodpaper{{../{pdf_file}}}{{{title}}}%
{{{authors}}}\n""".format(authors=texify(authors), pdf_file=pdfs[submission_id], title=texify(paper_title)), file=book_file)


# Write the volume-level bib with all the entries
dest_bib = f'proceedings/cdrom/{year}.{venue}-{volume}.bib'.format(year, venue, volume)
with open(dest_bib, 'w') as whole_bib:
    print('\n'.join(final_bibs), file=whole_bib)
    print('CREATED', dest_bib)

# Copy over the volume-level PDF
dest_pdf = dest_bib.replace('bib', 'pdf')
print('COPYING', full_pdf_file, '->', dest_pdf, file=sys.stderr)
copy(full_pdf_file, dest_pdf)



#!/usr/bin/env python3
from .utils              import readevent, easychair, archives as arin, searchid, sciencesconf
from .exports            import anthology, archives as arout, hal, dblp, pdf
from importlib.resources import files
from tomlkit             import dumps, parse
from zipfile             import ZipFile
import click, os, shutil, sys, subprocess, re, traceback

def print_help_msg(command):
    with click.Context(command) as ctx:
        click.echo(command.get_help(ctx))

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo('Version 1.0.0')
    ctx.exit()

def remove_dir(outd):
    for the_file in os.listdir(outd):
        file_path = os.path.join(outd, the_file)
        try:
            if os.path.isfile(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as exc:
            print(exc, file=sys.stderr)
    
def read_config():
    config_obj = None
    if not(os.path.exists('config.toml')):
        print('[Error] Configuration file config.toml not found, are you sure you are in a valid taln2x directory ?', file=sys.stderr)
        sys.exit(2)
    else:
        with open('config.toml', 'r') as config:
            config_text = config.read()
        config_obj = parse(config_text)
    return config_obj

@click.command()
@click.argument('action')
@click.argument('project_name', required=False)
@click.option('--in-format', default=None, \
              help='''input format''',
              type=click.Choice(['csv', 'xml', 'zip'], case_sensitive=False))
@click.option('--out-format', default=None, \
              help='''output format''',
              type=click.Choice(['pdf', 'acl', 'taln', 'hal', 'dblp'], \
                                case_sensitive=False))
@click.option('--verbose', '-v', default=None, type=int, \
              help='Define verbosity between 0 and 3.', show_default=True)
@click.option('--sessions', default=None, is_flag=True, \
              help='Do not ignore sessions defined in event.yml.')
@click.option('--tex-log', default=None, is_flag=True, \
              help='Do not remove logs of LaTeX compilation.')
@click.option('--stopwords', '-s', default=None, is_flag=True, \
              help='Ignore stopwords when sorting articles.')
@click.option('--ignore-pdf', default=None, is_flag=True, \
              help='Ignore metadata (abstract,keywords) from pdf ' + \
              'when these are available elsewhere.', show_default=True)
@click.option('--dump', default=None, is_flag=True, \
              help='Keep a copy of easy2acl\'s config file.')
@click.option('--xml', default=None, \
              help='''The name of the input XML file of the conference
              (TALN-archives input format).''')
@click.option('--zip-file', default=None, \
              help='''The name of the input ZIP file of the conference
              (sciencesconf input format).''')
@click.option('--anthology-id', default=None, \
              help='''The anthology ID of the conference.''')
@click.option('--bilingual', '-b', default=None, is_flag=True, \
              help='Generate bilingual article titles (cf anthology format).')
@click.option('--x-onbehalf', '-x', default=None, \
              help='''list of idHal_s (separated by ';') to be declared as co-owners
              during HAL ingestion (see https://aurehal.archives-ouvertes.fr/author).''')
@click.option('--stamp', default=None, \
              help='''Stamp (collection names, separated by ';') to be used during HAL
              export (see https://hal.science/browse/collection).''')
@click.option('--include-pdf', default=None, is_flag=True, \
              help='''During HAL ingestion, generate full exports made of references + pdf 
              files.''')
@click.option('--guess', '-g', default=None, is_flag=True, \
              help='''During HAL ingestion, activate the "guess affiliations".''')  
@click.option('--dry-run', '-d', default=None, is_flag=True, \
              help='''During HAL ingestion, use preprod server.''')  
@click.option('--halid', default=None, is_flag=True, \
              help='Include HAL id when compiling PDF proceedings.')
@click.option('--update', '-u', default=None, is_flag=True, \
              help='Update existing HAL reference/article.')
@click.option('--national', default=None, is_flag=True, \
              help='National event.', \
              show_default=True)
@click.option('--latex-encode', '-l', default=None, is_flag=True, \
              help='LaTeX-encode diacritics in output bibfiles.', \
              show_default=True)
@click.option('--no-meta', '-n', default=None, is_flag=True, \
              help='Do not miodify article metadata when updating HAL references.')
@click.option('--img', default=None, \
              help='Directory where to find user images.', \
              show_default=True)
@click.option('--lang', default=None, \
              help='Language used for pdf output.', \
              type=click.Choice(['fr', 'en'], case_sensitive=False), show_default=True)
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True, help='Display version number.')
def main(action, project_name, in_format, out_format, xml, zip_file, anthology_id, verbose, \
         bilingual, stopwords, ignore_pdf, dump, x_onbehalf, stamp, include_pdf, \
         guess, dry_run, halid, update, sessions, tex_log, no_meta, national, \
         latex_encode, img, lang):
    """Create proceedings (aka open archive) in various formats.

    ACTION should be one of : 'new' 'build' 'clean' 'list'

    In case of a *new* project, PROJECT_NAME is required (a directory of the corresponding name will be created)

    In case of a *build*, options are read from ./config.toml

    *list* is used to retrieve HAL identifiers of articles (if any)

    *clean* is used to remove output of previous conversions

    For more information, please consult https://talnarchives.gitlabpages.inria.fr/taln2x/
    """
    ############################
    ## INIT MODE
    ############################
    if action not in ['new', 'build', 'clean', 'list']:
        print("[Error] Unknown action (should be either new, convert, clean or list).", \
              file=sys.stderr)
        print("[Error] See also 'taln2x --help' for a full list of options.", \
              file=sys.stderr)        
        sys.exit(0)
    if action == 'new':
        if not(project_name):
            print("[Error] project name is missing\nPlease invoke taln2x init <project>", \
                  file=sys.stderr)
        else:
            in_dir= project_name
            if os.path.exists(in_dir):
                answer = input(f"Directory {in_dir} already exists, replace it ? [y/N]")
                if answer in ['yes', 'Yes', 'y', 'Y']:
                    shutil.rmtree(in_dir)                
                else:
                    click.echo("Abort.")
                    sys.exit(0)
            click.echo(f"\nCreating project directory {in_dir} ...")
            os.makedirs(in_dir)
            click.echo(f"Initializing project configuration in {in_dir}/config.toml ...")
            config_obj = parse(files('taln2x.templates').joinpath('config.toml').read_text())
            config_obj['project']['name'] = in_dir
            #print(dumps(config_obj))
            with open(os.path.join(in_dir, 'config.toml'), 'w') as configfile:
                print(dumps(config_obj), file=configfile)
            os.makedirs(os.path.join(in_dir, 'out'))            
            click.echo(f"Initializing event configuration file in {in_dir}/event.yml ...")
            shutil.copy(files('taln2x.templates').joinpath('event.yml'), in_dir)
            click.echo(f"Copying example logo and background pictures in {in_dir} ...")
            shutil.copy(files('taln2x.templates').joinpath('logo.png'), in_dir)
            shutil.copy(files('taln2x.templates').joinpath('background.png'), in_dir)
            shutil.copy(files('taln2x.templates').joinpath('by.eps'), in_dir)
            shutil.copy(files('taln2x.templates').joinpath('pre-proceedings.tex'), in_dir)
            shutil.copy(files('taln2x.templates').joinpath('pre-proceedings-en.tex'), in_dir)
            shutil.copy(files('taln2x.templates').joinpath('single_paper.tex'), in_dir)
            click.echo(f"Creating example track directory ({in_dir}/main) ...")
            os.makedirs(os.path.join(in_dir, 'main'))
            os.makedirs(os.path.join(in_dir, 'main', 'pdf'))
            shutil.copy(files('taln2x.templates').joinpath('articles.csv'), \
                        os.path.join(in_dir,'main'))
            shutil.copy(files('taln2x.templates').joinpath('articles.zip'), \
                        os.path.join(in_dir,'main', 'pdf'))
            with ZipFile(os.path.join(in_dir,'main', 'pdf', 'articles.zip'), mode='r') as zip_ref:
                zip_ref.extractall(path=os.path.join(in_dir,'main', 'pdf'))
            os.remove(os.path.join(in_dir,'main', 'pdf', 'articles.zip'))
            click.echo(f"Done.")
        sys.exit(0)
    elif action == 'clean':
        remove_dir('out')
        print('Cleaning output directory out/', file=sys.stderr)
        sys.exit(0)
    ############################
    ## DECLARATION (cf scope of variables)
    ############################
    ## Read config from command-line options
    # core options
    xin_format    = in_format
    xout_format   = out_format
    xverbose      = verbose
    xstopwords    = stopwords
    xignore_pdf   = ignore_pdf
    xsessions     = sessions
    xlatex_encode = latex_encode
    xdump         = dump 
    # csv options
    xid_col       = -1 #unset
    xauthors_col  = -1
    xtitle_col    = -1
    xidhal_col    = -1
    xfile_col     = -1
    xaccept_col   = -1
    xkeywords_col = -1
    # taln option
    xxml          = xml
    # sciencesconf option
    xzipfile      = zip_file
    # pdf options
    xtex_log      = tex_log
    ximg          = img
    xlang         = lang
    # acl options
    xanthology_id = anthology_id
    xbilingual    = bilingual
    # hal options
    xx_onbehalf   = x_onbehalf
    xstamp        = stamp
    xinclude_pdf  = include_pdf
    xguess        = guess
    xdry_run      = dry_run
    xhalid        = halid
    xupdate       = update
    xnational     = national
    xno_meta      = no_meta
    ## Update config from config file (note that CLI has priority over config file)
    if os.path.exists('config.toml'):
        config = read_config()
        # core options
        xin_format    = xin_format if xin_format else config['project']['core']['in_format']
        xout_format   = xout_format if xout_format else config['project']['core']['out_format']
        xverbose      = xverbose if xverbose else config['project']['core']['verbose']
        xstopwords    = xstopwords if xstopwords else config['project']['core']['stopwords']
        xignore_pdf   = xignore_pdf if xignore_pdf else config['project']['core']['ignore_pdf']
        xsessions     = xsessions if xsessions else config['project']['core']['sessions']
        xlatex_encode = xlatex_encode if xlatex_encode else config['project']['core']['latex_encode'] 
        xdump         = xdump if xdump else config['project']['core']['dump']
        # csv options
        xid_col       = config['project']['csv']['id_col']
        xauthors_col  = config['project']['csv']['authors_col']
        xtitle_col    = config['project']['csv']['title_col']
        xidhal_col    = config['project']['csv']['idhal_col']
        xfile_col     = config['project']['csv']['file_col']        
        xaccept_col   = config['project']['csv']['accept_col']
        xkeywords_col = config['project']['csv']['keywords_col']
        # taln option
        xxml          = xxml if xxml else config['project']['xml']['xml_file']
        # sciencesconf option
        xzipfile      = xzipfile if xzipfile else config['project']['zip']['zip_file']
        # pdf options
        xtex_log      = xtex_log if xtex_log else config['project']['pdf']['tex_log']
        ximg          = ximg if ximg else config['project']['pdf']['img']
        xlang         = xlang if xlang else config['project']['pdf']['language']
        # acl options
        xanthology_id = xanthology_id if xanthology_id else config['project']['acl']['anthology_id']
        xbilingual    = xbilingual if xbilingual else config['project']['acl']['bilingual']
        # hal options
        conf_on_behalf= config['project']['hal']['x_onbehalf'] if config['project']['hal']['x_onbehalf'] != '' else None
        xx_onbehalf   = xx_onbehalf if xx_onbehalf else conf_on_behalf 
        conf_stamp    = config['project']['hal']['stamp'] if config['project']['hal']['stamp'] != '' else None
        xstamp        = xstamp if xstamp else conf_stamp
        xinclude_pdf  = xinclude_pdf if xinclude_pdf else config['project']['hal']['include_pdf'] 
        xguess        = xguess if xguess else config['project']['hal']['guess'] 
        xdry_run      = xdry_run if xdry_run else config['project']['hal']['dry_run'] 
        xhalid        = xhalid if xhalid else config['project']['hal']['halid'] 
        xupdate       = xupdate if xupdate else config['project']['hal']['update'] 
        xnational     = xnational if xnational else config['project']['hal']['national']         
        xno_meta      = xno_meta if xno_meta else config['project']['hal']['no_meta'] 

    if not(os.path.exists('event.yml')):
        print("[Error] event.yml configuration file not found.", \
              file=sys.stderr)
        sys.exit(1)

    e      = readevent.get_meta('event.yml', xanthology_id)  #event
    indir  = os.getcwd()
    outdir = os.path.join(indir, 'out')

    ############################
    ## CASE: EASYCHAIR INPUT
    ############################
    if xin_format == 'csv':
        print('\nReading event information\n=========================', file=sys.stderr)
        easychair.get_data(e, indir, xverbose, xignore_pdf, xsessions, \
                           xid_col, xauthors_col, xtitle_col, xidhal_col, xfile_col, \
                           xaccept_col, xkeywords_col)
        if xverbose > 1:
            print(e)
    ############################
    ## CASE: TALN-ARCHIVES INPUT
    ############################lan
    elif xin_format == 'xml':
        if not(os.path.exists(xxml)) :
            print('\n[Error] Missing input XML file, exit.\n', file=sys.stderr)
            print_help_msg(main)
            exit(1)
        else:
            print('\nReading XML event file\n=========================', file=sys.stderr)
            try:
                arin.read_xml(e, xxml)
            except ValueError:
                print('[Error] Cannot read input XML file ' + xxml + ', abort.', file=sys.stderr)
                exit(20)
        if xverbose > 1:
            print(e)
    ############################
    ## CASE: SCIENCESCONF INPUT
    ############################
    elif xin_format == 'zip':
        if not(os.path.exists(xzipfile)) :
            print(f'\n[Error] {xzipfile} not found. Missing input ZIP file. Abort.\n', file=sys.stderr)
            print_help_msg(main)
            exit(1)
        else:
            print(f'\nReading ZIP file {xzipfile}\n=========================', file=sys.stderr)
            try:
                # a) convert sciencesconf's zip file into easychair format
                sciencesconf.get_data(e, xzipfile, indir, xverbose, xid_col, xauthors_col, xtitle_col, \
                                      xidhal_col, xfile_col, xaccept_col, xkeywords_col)
                # b) read easychair-like data
                easychair.get_data(e, indir, xverbose, xignore_pdf, xsessions, \
                           xid_col, xauthors_col, xtitle_col, xidhal_col, xfile_col, \
                           xaccept_col, xkeywords_col)
            except ValueError as ve:
                traceback.print_exc()
                print('[Error] Cannot read input ZIP file ' + xzipfile + ', abort.', file=sys.stderr)
                exit(20)
        if xverbose > 1:
            print(e)
    else:
        print('[Error] Unknown input format', file=sys.stderr)
        sys.exit(12)

    ############################
    ## ACTUAL ACTION
    ############################
    if action == 'build':
        if not(os.path.exists('event.yml')): #in case we call taln2x from a wrong dir
            print('\n[Error] Missing event yaml configuration file, exit.\n', \
                  file=sys.stderr)
            print_help_msg(main)
            sys.exit(12)
        else:
            click.echo("\nConverting directory ...")
    elif action == 'list' :
        # Listing existing HAL ids (for manual checking)
        print('\nExtracting HAL identifiers\n=========================', file=sys.stderr)
        if shutil.which('curl') is None:
            print('[Error] You need to install \"curl\" to use the --list-id option.', \
                  file=sys.stderr)
        else:
            searchid.get_id(e, os.getcwd(), xstopwords, xverbose, xsessions)
        sys.exit(0)
    else:
        click.echo("\nUnknown ACTION - please invoke 'taln2x --help' \n")
        sys.exit(1)

    ############################
    ## CASE: PDF EXPORT
    ############################
    if xout_format == 'pdf':
        if xhalid:
            if shutil.which('curl') is None:
                print('[Error] You need to install \"curl\" to use the --halid option.', \
                      file=sys.stderr)
                sys.exit(197)
        print('\nGenerating PDF proceedings\n=========================', file=sys.stderr)
        pdf.write_tex(e, xstopwords, xverbose, xhalid, xsessions, xtex_log, ximg, xlang)
    ############################
    ## CASE: EASY2ACL EXPORT
    ############################
    elif xout_format == 'acl':
        print('\nGenerating ACL anthology files\n=========================', file=sys.stderr)
        shutil.copy(files('taln2x.templates').joinpath('easy2acl.py'), outdir)
        shutil.copy(files('taln2x.templates').joinpath('blank.pdf'), outdir)
        ## actual compilation
        anthology.easy2acl_export(e, xbilingual, xstopwords, xverbose, xdump, xsessions, \
                                  xlatex_encode)
        ## cleaning
        shutil.rmtree(os.path.join(outdir, 'easy2acl'))
        os.unlink(os.path.join(outdir, 'easy2acl.py'))
        os.unlink(os.path.join(outdir, 'blank.pdf'))
    ############################
    # CASE: TALN ARCHIVES EXPORT
    ############################
    elif xout_format == 'taln':
        print('\nGenerating event XML file\n=========================', file=sys.stderr)
        arout.write_xml(e, xstopwords, xverbose, xdump, xsessions, xlatex_encode)
    ############################
    # CASE: HAL ARCHIVES EXPORT
    ############################
    elif xout_format == 'hal':
        shutil.copy(files('taln2x.templates').joinpath('aofr.xsd'), outdir)
        print('\nGenerating event HAL BIB and ZIP files\n=========================', \
              file=sys.stderr)
        hal.write_hal_xml(e, xstopwords, xverbose, xx_onbehalf, xstamp, xinclude_pdf, \
                          xguess, xdry_run, xupdate, xsessions, xno_meta, xnational)
        print('\nSWORD shell scripts created, to use them, please export LOGIN and ' + \
              'PASSWORD environment variables containing your HAL credentials.\n', \
              file=sys.stderr)
        os.unlink(os.path.join(outdir, 'aofr.xsd'))
    ############################
    # CASE: DBLP ARCHIVES EXPORT
    ############################
    elif xout_format == 'dblp':
        shutil.copy(files('taln2x.templates').joinpath('dblpsubmission.dtd'), outdir)
        print('\nGenerating event DBLP XML files\n=========================', file=sys.stderr)
        dblp.write_dblp_xml(e, xstopwords, xverbose, xsessions)
        os.unlink(os.path.join(outdir, 'dblpsubmission.dtd'))
    ############################
    else:
        print("[Error] Unknown output format", file=sys.stderr)
        sys.exit(8)

if __name__ == '__main__':
    main()



class Event(object):

    def __init__(self):
        self.title          = ''
        self.abbrev         = ''
        self.chairs         = []
        self.location       = ''
        self.booktitle      = ''
        self.shortbooktitle = ''
        self.begin          = None #datetime type
        self.end            = None #datetime type
        self.month          = ''
        self.url            = ''
        self.publisher      = ''
        self.sig            = ''
        self.base_url       = '' #for TALN archives
        self.anthology      = ''
        self.tracks         = {}   #dict mapping track id with track object (formerly itself a dict)

    def __str__(self):
        s = ''
        for k,v in self.__dict__.items(): 
            if k != 'tracks':
                s += k + ':' + str(v) + '\n'
            else:
                for t,i in v.items():
                    s += '\n' + t + '\n'
                    for a,b in i.__dict__.items():
                        if type(b) == list: #articles
                            s += a + ':' + '\n  '.join(list(map(lambda x: str(x), b))) + '\n'
                        elif type(b) == dict: #authors
                            flat_list = [item for sublist in b.values() for item in sublist]
                            s += a + ':' + '\n  '.join(list(map(lambda x: str(x), flat_list))) + '\n'
                        else:
                            s += a + ':' + str(b) + '\n'
        return s



import yaml
from .           import Event, Track

def get_meta(config, anthology_id):
    event = Event()
    doc = {}
    with open(config, 'r') as f:
        try:
            doc = yaml.safe_load(f)
        except yaml.YAMLError as exc:
            print(exc)
            pm = exc.problem_mark
            print("Your file {} has an issue on line {} at position {}".
                  format(pm.name, pm.line, pm.column))
        #print(doc)
    # Feed an Event object
    for k,v in doc.items():
        #print(k,v)
        if k == 'tracks': #v is then a dict mapping trackid with a dict
            for x in v.keys():
                t = Track(x)
                t.volume = x  # Add volume name (for anthology)
                for y in v[x].keys(): # Turn track's dict into an object
                    t.__dict__[y] = v[x][y]
                event.__dict__['tracks'][x] = t
        else:
            event.__dict__[k] = v
    # Command line flag anthology overrides config file's value
    if anthology_id is not None:
        event.__dict__['anthology'] = anthology_id
    # Auto-completions: (mandatory fields for anthology)
    if event.__dict__['shortbooktitle'] == '':
        if 'short_booktitle' in event.__dict__:
            event.__dict__['shortbooktitle'] = event.__dict__['short_booktitle']
        else:
            event.__dict__['shortbooktitle'] = event.__dict__['booktitle']
    if event.__dict__['begin'] is not None :
        event.__dict__['month'] = event.__dict__['begin'].month
        event.__dict__['year']  = event.__dict__['begin'].year
    # Convenience for LaTeX-PDF export (newlines in title)
    if '\\' in event.__dict__['booktitle']:
        event.__dict__['texbooktitle'] = event.__dict__['booktitle']
        event.__dict__['booktitle']    = event.__dict__['booktitle'].\
                                         replace('\\','')
    return event



import os, shutil, re, sys, tqdm
import xml.etree.cElementTree as ET
from xml.dom             import minidom
from unidecode           import unidecode
from lxml                import etree
from ..exports.anthology import order_papers, order_session

DBLP_ID  = 'conf/taln'

def write_dblp_xml(event, stopwords, verbose, sessions):
    indir   = os.getcwd()
    xmldir  = os.path.join(indir, "out")
    # Preparing output dir
    eventname= 'dblp_' + unidecode(event.__dict__['abbrev']) + \
               '-' + str(event.__dict__['year'])
    if not os.path.exists(os.path.join(xmldir, eventname)):
        os.mkdir(os.path.join(xmldir, eventname))
    ## Process articles
    for i,t in enumerate(event.__dict__['tracks']):
        print('Processing track:', t, file=sys.stderr)
        #  Base url
        base_url = os.path.join('http://talnarchives.atala.org', \
                                event.__dict__['abbrev'], \
                                event.__dict__['abbrev'] +'-'+ str(event.__dict__['year'])) 
        if event.__dict__['tracks'][t].base_url != '':
            base_url = event.__dict__['tracks'][t].__dict__['base_url']
        else:
            print('[Warning] No base_url defined in track configuration (event.yml)\n' + \
                  'Default (TALN archives) URL used instead, please check output', \
                  file=sys.stderr)
        volumename = event.__dict__['booktitle'] + '. ' + \
                     event.__dict__['tracks'][t].__dict__['fullname'] 
        xmlfile  = os.path.join(xmldir, eventname, \
                                eventname.lower() + '-' + str(i+1) + '.xml')
        # Preparing XML content (DOM)
        root     = ET.Element('dblpsubmission')
        proc     = ET.SubElement(root, 'proceedings')
        ## Process edition
        ET.SubElement(proc, 'key').text = DBLP_ID
        for c in event.__dict__['chairs']:
            ET.SubElement(proc, 'editor').text = c.split(",")[1].strip() + " " + \
                                                 c.split(",")[0].strip()
        ET.SubElement(proc, 'title').text = volumename
        if event.__dict__['publisher'] != '':
            ET.SubElement(proc, 'publisher').text = event.__dict__['publisher']
        ET.SubElement(proc, 'year').text = str(event.__dict__['year'])
        conf = ET.SubElement(proc, 'conf')
        ET.SubElement(conf, 'acronym').text = event.__dict__['abbrev']
        ET.SubElement(conf, 'location').text= event.__dict__['location']
        span = str(event.__dict__['begin'].day) + '-' + str(event.__dict__['end'].day)
        if event.__dict__['begin'].day == event.__dict__['end'].day:
            span = str(event.__dict__['begin'].day)
        ET.SubElement(conf, 'date').text    = months[event.__dict__['begin'].month] + ' ' +\
                                              span + ', ' +\
                                              str(event.__dict__['end'].year)
        ET.SubElement(conf, 'url').text     = event.__dict__['url']
        toc = ET.SubElement(proc, 'toc')
        ## Processing papers
        # track's config overrides command line one
        order   = 'input' #default
        if event.__dict__['tracks'][t].order != '':
            order = event.__dict__['tracks'][t].__dict__['order']
        papers  = event.__dict__['tracks'][t].__dict__['articles']
        ordered = papers
        if sessions and event.__dict__['tracks'][t].sessions != '':
            ordered = order_session(papers, order, stopwords, verbose, \
                                    event.__dict__['tracks'][t])
        else:
            ordered = order_papers(papers, order, stopwords, verbose, \
                                   event.__dict__['tracks'][t].__dict__['authors'])
        # Track settings overwrite cli start option
        start    = 1 #default
        if int(event.__dict__['tracks'][t].startpage) > 0:
            start = int(event.__dict__['tracks'][t].__dict__['startpage'])
        current_pos = start
        for art in tqdm.tqdm(ordered):
            if art:
                ## Publication
                article = ET.SubElement(toc, 'publ')
                try:
                    for au in event.__dict__['tracks'][t].__dict__['authors'][int(art.__dict__['paperid'])]:
                        ET.SubElement(article, 'author').text = au.__dict__['firstname']+' '+\
                                                                au.__dict__['lastname']
                except KeyError:
                    for aut2 in art.__dict__['authors'].replace(' and ', ', ').split(', '):
                        aut     = re.sub(r'([^\\])~', r'\1 ', aut2)
                        fname   = aut.split(' ')[0]
                        lname   = ' '.join(aut.split(' ')[1:])
                        ET.SubElement(article, 'author').text = fname + ' ' + lname
                ## Title
                ET.SubElement(article, 'title').text = art.__dict__['title']
                pages = art.__dict__['pages']
                if pages == '':
                    pages = str(current_pos) + '-' + str(current_pos + art.__dict__['numpages']-1)
                current_pos += art.__dict__['numpages']
                ET.SubElement(article, 'pages').text=pages
                ET.SubElement(article, 'ee').text=os.path.join(base_url, \
                                                               art.__dict__['paperid']+'.pdf')
            else:
                current_pos += 1 # skip session's delimiter
        ## Printing and validating XML file
        xmltree = etree.fromstring(ET.tostring(root, short_empty_elements=False))
        dtd = etree.DTD(os.path.join(indir, 'out', 'dblpsubmission.dtd'))
        if not dtd.validate(xmltree):
            print('XML DTD validation error', file=sys.stderr)
            print(dtd.error_log.filter_from_errors(), file=sys.stderr)
            sys.exit(1)
        with open(xmlfile, 'w') as f:
            print(etree.tostring(xmltree, encoding="utf8", xml_declaration=True, \
                                 pretty_print=True, \
                                 doctype=\
                                 '<!DOCTYPE dblpsubmission SYSTEM "dblpsubmission.dtd">').\
                  decode(), file=f)
